package com.inxar.jenesis;

/**
 * $Id: Jikes.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
import java.io.*;
import java.util.Vector;
import java.util.StringTokenizer;
import org.inxar.jenesis.*;
import com.inxar.jenesis.*;
import com.inxar.jenesis.util.*;

/**
 * Implementation of <code>VirtualCompiler</code> which makes a system
 * call to jikes.  
 */
public class Jikes
	extends MVC
{
    /**
     * Constructs a new Jikes compiler instance.
     */
    public Jikes()
    {
	super();
    }
    public boolean compile(CompilationUnit unit) throws java.io.IOException
    {
	// Get the path separator (unix = ':', dos = ';').
	String sep = System.getProperty("path.separator");

	StringBuffer cmd = new StringBuffer("jikes ");

	// Then add the destination
	if (dest != null)
	    cmd.append(" -d ").append(dest);

	// Append all the sourcepath arguments
	if (srcs.size() > 0) {
	    cmd.append(" -sourcepath");
	    for (int i = 0; i < srcs.size(); i++) {
		if (i > 0) cmd.append(sep);
		cmd.append((String)libs.elementAt(i));
	    }
	}

	// Then finally do the classpath
        cmd.append(" -classpath ").append(unit.getCodebase());

	// Append all the classpath arguments
	for (int i = 0; i < libs.size(); i++)
	    cmd.append(sep).append((String)libs.elementAt(i));

	// Start off with adding the options
	for (int i = 0; i < opts.size(); i++)
	    cmd.append((String)opts.elementAt(i));

        // Add the filename of the unit to compile.
	cmd.append(' ').append(toFilename(unit));

	// Now we exec this off the runtime and read the results.
	String result = execute(cmd.toString());
	
	if (result != null && result.length() > 0) {
	    System.out.println(result);
	    return false;
	} else {
	    return true;
	}
	    
    }
    protected String execute(String cmd)
    {
	Runtime r = Runtime.getRuntime();
	Process p = null;
	BufferedInputStream in = null;
	StringWriter out = null;
 
	// execute and get the process.
	try {
 
	    // Run it...
	    p = r.exec(cmd);
 
	    in = new BufferedInputStream(p.getErrorStream());
	    out = new StringWriter();
 
	    // Read fully
	    int c;
	    while ((c = in.read()) != -1)
		out.write(c);
 
	    // log
	    out.close();
	    in.close();

	    return out.toString();
 
	} catch (Exception ex) {
 
	    ex.printStackTrace();
	    return out == null ? null : out.toString();
 
	} finally {
 
	    if (out != null) try { out.close(); } catch (Exception e) {}
	    if (in != null)  try { in.close();  } catch (Exception e) {}
 
	}
    }
}
