package com.inxar.jenesis;
/**
 *
 * Copyright (C) 2008  Richard van Nieuwenhoven - ritchie [at] gmx [dot] at
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.inxar.jenesis.Annotation;
import org.inxar.jenesis.AnnotationAttribute;
import org.inxar.jenesis.ArrayType;
import org.inxar.jenesis.CodeWriter;
import org.inxar.jenesis.Expression;

import com.inxar.jenesis.MVM.MCodeable;

public class MAnnotation extends MCodeable implements Annotation {

	static class MAnnotationAttribute extends MCodeable implements
			org.inxar.jenesis.AnnotationAttribute {

		MAnnotationAttribute(MVM vm) {
			super(vm);
		}

		String name;

		Expression[] values;

		boolean isArray;

		Annotation[] valueAnnotations;

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public Expression[] getValues() {
			return values;
		}

		public void setValues(Expression[] values) {
			this.values = values;
		}

		public Annotation[] getValueAnnotations() {
			return valueAnnotations;
		}

		public void setValueAnnotations(Annotation[] valueAnnotations) {
			this.valueAnnotations = valueAnnotations;
		}

		public Expression getValue() {
			if (this.values != null && this.values.length > 0) {
				return this.values[0];
			} else {
				return null;
			}
		}

		public Annotation getValueAnnotation() {
			if (this.valueAnnotations != null
					&& this.valueAnnotations.length > 0) {
				return this.valueAnnotations[0];
			} else {
				return null;
			}
		}

		public void setValue(Expression value) {
			this.values = new Expression[] { value };
		}

		public void setValueAnnotation(Annotation valueAnnotation) {
			this.valueAnnotations = new Annotation[] { valueAnnotation };
		}

		public void addValueAnnotation(Annotation valueAnnotation) {
			Annotation[] oldValue = this.valueAnnotations;
			if (oldValue == null) {
				oldValue = new Annotation[0];
			}
			this.valueAnnotations = new Annotation[oldValue.length + 1];
			System.arraycopy(oldValue, 0, this.valueAnnotations, 0,
					oldValue.length);
			this.valueAnnotations[oldValue.length] = valueAnnotation;
		}

		public void addValue(Expression value) {
			Expression[] oldValue = this.values;
			if (oldValue == null) {
				oldValue = new Expression[0];
			}
			this.values = new Expression[oldValue.length + 1];
			System.arraycopy(oldValue, 0, this.values, 0, oldValue.length);
			this.values[oldValue.length] = value;
		}

		public CodeWriter toCode(CodeWriter out) {
			if (!isArray() && valueAnnotations != null
					&& valueAnnotations.length == 1) {
				out.indentLine();
			}
			if (name != null && name.trim().length() > 0) {
				out.write(name);
				out.write(" = ");
			}
			if (isArray()) {
				out.write('{');
				out.indentLine();
			}
			if (values != null) {
				for (int index = 0; index < values.length; index++) {
					if (index > 0) {
						out.write(", ");
					}
					out.write(values[index]);
				}
			} else if (valueAnnotations != null) {
				for (int index = 0; index < valueAnnotations.length; index++) {
					if (index > 0) {
						out.write(", ");
						out.newLine();
					}
					out.write(valueAnnotations[index]);
				}
			}
			if (isArray()) {
				out.dedentLine();
				out.write('}');
			} else if (valueAnnotations != null && valueAnnotations.length == 1) {
				out.dedentLine();
			}
			return out;
		}

		public boolean isArray() {
			return isArray
					|| (values != null && values.length > 1)
					|| (valueAnnotations != null && valueAnnotations.length > 1);
		}

		public void setArray(boolean isArray) {
			this.isArray = isArray;
		}
	}

	String name;

	String text;

	List annotationAttributes = new ArrayList();

	MAnnotation(MVM vm, String text) {
		super(vm);
		this.text = text;
	}

	MAnnotation(MVM vm, String name, String text) {
		super(vm);
		this.name = name;
		this.text = text;
	}

	public CodeWriter toCode(CodeWriter out) {
		super.toCode(out);
		if (annotationAttributes.size() == 0
				&& (name == null || "".equals(name))) {
			out.write("@" + text);
		} else if (annotationAttributes.size() == 0) {
			out.write("@" + name + "(" + text + ")");
		} else {
			out.write("@");
			if (name != null && name.trim().length() > 0) {
				out.write(name);
			} else if (text != null && text.trim().length() > 0) {
				out.write(text);
			}
			out.write("(");
			for (Iterator iterator = annotationAttributes.iterator(); iterator
					.hasNext();) {
				out.write(iterator.next());
				if (iterator.hasNext()) {
					out.write(", ");
				}
			}
			out.write(")");
		}
		return out;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getText() {
		return text;
	}

	public void setText(String text) {
		this.text = text;
	}

	public List getAnnotationAttributes() {
		return annotationAttributes;
	}

	public void addAnntationAttribute(AnnotationAttribute annotation) {
		annotationAttributes.add(annotation);
	}

	public AnnotationAttribute addAnntationAttribute(String name) {
		MAnnotationAttribute attribute = new MAnnotationAttribute(vm);
		attribute.setName(name);
		addAnntationAttribute(attribute);
		return attribute;
	}

	public AnnotationAttribute addAnntationAttribute(String name,
			Expression value) {
		AnnotationAttribute attribute = addAnntationAttribute(name);
		attribute.setValue(value);
		return attribute;
	}

	public AnnotationAttribute getAnnotationAttribute(String name) {
		for (Iterator iterator = annotationAttributes.iterator(); iterator
				.hasNext();) {
			AnnotationAttribute annotationAttribute = (AnnotationAttribute) iterator
					.next();
			if (annotationAttribute.getName().equals(name)) {
				return annotationAttribute;
			}
		}
		return null;
	}
}
