package com.inxar.jenesis;

/**
 * $Id: MComment.java,v 1.2 2003-08-25 15:53:08 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
import java.util.Vector;
import org.inxar.jenesis.*;
import com.inxar.jenesis.util.*;

/**
 * Standard <code>Comment</code> implementations.
 */
public abstract class MComment
	extends MVM.MCodeable
	implements Comment
{
	public static final int S_COMMENT_LENGTH = 30;
	public static final int M_COMMENT_LENGTH = 180;
	public static final int D_COMMENT_LENGTH = 180;

	public static final int COMMENT_LENGTH_UNLIMITED = 500;
 

	int type;
	String text;

	/* =============================================================== */
	/* SINGLE LINE COMMENT */
	/* =============================================================== */
	static final class MSingleLineComment
	extends MComment
	{
	MSingleLineComment(MVM vm, String text)
	{
	    super(vm, Comment.S, text);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);

	    MComment.wordWrap(text, S_COMMENT_LENGTH, "//", out);

	    return out;
	}
	}

	/* =============================================================== */
	/* MULTIPLE LINE COMMENT */
	/* =============================================================== */
	static final class MMultipleLineComment
	extends MComment
	{
		
	boolean onlyAtBeginningAndEnd = false;
	
	MMultipleLineComment(MVM vm, String text)
	{
	    super(vm, Comment.M, text);
	}

	MMultipleLineComment(MVM vm, String text, boolean onlyAtBeginningAndEnd)
	{
		super(vm, Comment.M, text);
		this.onlyAtBeginningAndEnd = onlyAtBeginningAndEnd; 
		
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);

	    if (!out.isLineNew()) out.newLine();
	    out.write("/* ").newLine(); // */
	    if (onlyAtBeginningAndEnd)
			MComment.wordWrap(text, COMMENT_LENGTH_UNLIMITED, "\t", out);
	    else
			MComment.wordWrap(text, M_COMMENT_LENGTH, " *", out);
	    
	    if (!out.isLineNew()) out.newLine();
	    out.write(" */");

	    return out;
	}
	}

	/* =============================================================== */
	/* DOCUMENTATION COMMENT */
	/* =============================================================== */
	static class MDocumentationComment
	extends MComment
	implements DocumentationComment
	{
	public MDocumentationComment(MVM vm, String text)
	{
	    super(vm, Comment.D, text);
	    vtagParameter = new Vector();
	    vtagSee = new Vector();
	}

	public void setAuthor(String s)
	{
	    this.tagAuthor = s;
	}

	public void setDate(String s)
	{
	    this.tagDate = s;
	}

	public void setDeprecated(String s)
	{
	    this.tagDeprecated = s;
	}

	public void setException(String s)
	{
	    this.tagException = s;
	}

	public void setReturn(String s)
	{
	    this.tagReturn = s;
	}

	public void setSerial(String s)
	{
	    this.tagSerial = s;
	}

	public void setSerialData(String s)
	{
	    this.tagSerialData = s;
	}

	public void setSerialField(String s)
	{
	    this.tagSerialField = s;
	}

	public void setSince(String s)
	{
	    this.tagSince = s;
	}

	public void setThrows(String s)
	{
	    this.tagThrows = s;
	}

	public void setVersion(String s)
	{
	    this.tagVersion = s;
	}

	public String getAuthor()
	{
	    return tagAuthor;
	}

	public String getDate()
	{
	    return tagDate;
	}

	public String getDeprecated()
	{
	    return tagDeprecated;
	}

	public String getException()
	{
	    return tagException;
	}

	public String getReturn()
	{
	    return tagReturn;
	}

	public String getSerial()
	{
	    return tagSerial;
	}

	public String getSerialData()
	{
	    return tagSerialData;
	}

	public String getSerialField()
	{
	    return tagSerialField;
	}

	public String getSince()
	{
	    return tagSince;
	}

	public String getThrows()
	{
	    return tagThrows;
	}

	public String getVersion()
	{
	    return tagVersion;
	}

	public Iterator getParams()
	{
	    return new EnumerationIterator(vtagParameter.elements(), vtagParameter.size());
	}

	public void addParam(String s)
	{
	    vtagParameter.addElement(s);
	}

	public Iterator getSees()
	{
	    return new EnumerationIterator(vtagSee.elements(), vtagSee.size());
	}

	public void addSee(String s)
	{
	    vtagSee.addElement(s);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);

	    if (!out.isLineNew()) out.newLine();
	    out.write("/**").newLine(); // */

	    MComment.wordWrap(text, D_COMMENT_LENGTH, " *", out);

	    int lineNo = out.getLineNumber();

	    writeTag(tagAuthor, "author", out);
	    writeTag(tagDate, "date", out);
	    writeTag(tagDeprecated, "deprecated", out);
	    writeTag(tagException, "exception", out);
	    writeTag(tagReturn, "return", out);
	    writeTag(tagSerial, "serial", out);
	    writeTag(tagSerialData, "serialData", out);
	    writeTag(tagSerialField, "serialField", out);
	    writeTag(tagSince, "since", out);
	    writeTag(tagThrows, "throws", out);
	    writeTag(tagVersion, "version", out);
	    writeTags(vtagParameter, "param", out);
	    writeTags(vtagSee, "see", out);

	    if (!out.isLineNew()) out.newLine();
	    out.space().write("*/");

	    return out;
	}

	private void writeTag(String tag, String tagLabel, CodeWriter out)
	{
	    if (tag == null)
		return;

	    String s = new StringBuffer()
		.append('@')
		.append(tagLabel)
		.append(' ')
		.append(tag)
		.toString();

	    MComment.wordWrap(s, 55, " *", out);
	}

	private void writeTags(Vector v, String tagLabel, CodeWriter out)
	{
	    int len = v.size();

	    for (int i=0; i<len; i++)
		writeTag((String)v.elementAt(i), tagLabel, out);
	}

	String tagAuthor;
	String tagDate;
	String tagDeprecated;
	String tagException;
	String tagReturn;
	String tagSerial;
	String tagSerialData;
	String tagSerialField;
	String tagSince;
	String tagThrows;
	String tagVersion;
	Vector vtagParameter;
	Vector vtagSee;
	}

    MComment(MVM vm, int type, String text)
    {
	super(vm);
	this.type = type;
	this.text = text;
    }
    public String getText()
    {
	return text;
    }
    public void setText(String text)
    {
	this.text = text;
    }
    public int type()
    {
	return type;
    }
    static void wordWrap(char[] s, int maxlen, String head, CodeWriter out)
    {
	int strlen = s.length, trailer = 0, leader = 0, anchor = 0;

	// loop while we look for newline characters.
        while (leader < strlen) {

            // are we at the max line length?  If so then we need
            // to write the line thus far (but do so at a good place)
            if (leader - trailer == maxlen) {
				// set the anchor in case we need to retrieve our position
		anchor = leader;
				// we don't want words to be broken up.
				// check to make sure the leader hits a space OR a comma (added comma 9/5/00)
	    word_search: while (s[leader] == ',' || s[leader] != ' ') {
				//word_search: while (s[leader] != ' ') {
		// move back one character
		--leader;
		// if it goes all the way back, then too bad, we have to cut it...
		if (leader == trailer) {
		    leader = anchor;
		    break word_search;
		}
	    }
				// ok, we've discovered a place to break the line.  Now we write
				// this much of the output and reset the tracking variables.
		out.write(head).space().write(s, trailer, leader - trailer).newLine();

            	// move the trailer up to the leader
            	trailer = leader;
            }

            // if we've gone this far we need to just make sure that the character is
            // not a newline
            switch (s[leader]) {
	    case 13: /* CR */ {
		// in this case we check to see if there is an LF after
		// this character.  If so this is a Caldera newline
		if (s[leader + 1] == 10) {
		    // flush the line thus far
		    out.write(head).space().write(s, trailer, leader - trailer).newLine();
		    // move the leader up two characters such that we skip over the newline
		    trailer = leader + 2;
		    leader = leader + 2;
                    // looks like a mac newline.  Whadda know?
		} else {
		    // flush the line thus far
		    out.write(head).space().write(s, trailer, leader - trailer).newLine();
		    // move the leader up one character such that we skip over the newline
		    trailer = ++leader;
		}
		break;
	    } case 10: /* LF */ {
		// this is a nix machine
		// flush the line thus far
		out.write(head).space().write(s, trailer, leader - trailer).newLine();
		// move the leader up one character such that we skip over the newline
		trailer = ++leader;
		// bust out
		break;
	    } default: {
		// just advance the leader
		leader++;
	    }
            }
	}

       	// flush the rest of the line to the writer
	out.write(head).space().write(s, trailer, leader - trailer).newLine();
    }
    /* =============================================================== */
    /* CLASS METHODS */
    /* =============================================================== */

    /**
     * <P>Breaks the given String into lines with given length int
     * tabbed to a distance given int tab and prefixed with an asterisk
     * and a space. Therefore, <P>
     *
     * <code>wordWrap("How now brown cow", 5, 0);</code>
     *
     * <P>returns:<P>
     *
     *  * How<BR>
     *  * now<BR>
     *  * brown<BR>
     *  * cow<BR>
     */
    static void wordWrap(String s, int maxlen, String head, CodeWriter out)
    {
	wordWrap(s.toCharArray(), maxlen, head, out);
    }
}
