package com.inxar.jenesis;

/**
 * $Id: MDeclaration.java,v 1.6 2006-02-16 21:59:50 wee Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
import java.io.*;
import java.util.Vector;
import java.util.Comparator;
import org.inxar.jenesis.*;

import com.inxar.jenesis.util.*;

/**
 * Standard <code>Declaration</code> implementations.
 */
abstract class MDeclaration
	extends MVM.MCodeable
{
	// ===============================================================
	// COMPILATION UNIT DECLARATION
	// ===============================================================
	static class MCompilationUnit
	extends MDeclaration
	implements CompilationUnit
	{
	MCompilationUnit(MVM vm, String codebase)
	{
		super(vm);
		this.codebase = codebase;
		types = new Vector();
		imports = new Vector();
	}

	public String getFileName()
	{
		// make the fileName of the target file as the concatenation
		// of the sourcepath, the package name, and the className,
		// and the ending dot java
		StringBuffer file = null;

		// was a path specified?
		if (codebase != null && codebase.length() > 0) {
		// make the buffer
		file = new StringBuffer(codebase);
		// does the codebase end with a slash?
		if (!codebase.endsWith(new Character(File.separatorChar).toString()))
			// make it so...
			file.append(File.separatorChar);
		} else {
		file = new StringBuffer();
		}

		// append to the filepath if exists
		if (namespace != null) { 
		// fetch the package name
		String pkg = namespace.getName();
		if (pkg != null) {
			// append the package name
			file.append(pkg.replace('.', java.io.File.separatorChar));
		}
		}

		// add a separator, the top level name, and the extention
		file.append(java.io.File.separatorChar)
		.append(this.getTopLevelType().getName())
		.append(".java");


		// done
		return file.toString();
	}

	public void encode() throws java.io.IOException
	{
		// have we already called this method?
		if (isEncoded)
		return;

		// wrap this section for io exceptions
		PrintWriter fout = null;

		try {

		// Make a file object which corresponds to the
		// filename which we will write to.
		File file = new File(getFileName());

		// Get the parent file such that we can create the
		// directory if necessary. NOTE: JDK1.2 dependency
		// here.
		File dir = file.getParentFile();

		// Make sure it exists.
		if (!dir.exists())
			dir.mkdirs();

		// now we want to make the stream...
		fout = new PrintWriter
			(new BufferedWriter
			(new FileWriter(file)));

		// ...wrap it with a code writer...
		CodeWriter out = new MCodeWriter(fout);

		// ...generate the code...
		this.toCode(out);

		// ... and finally close and drop the stream
		fout.close(); fout = null;

		isEncoded = true;

		} catch (IOException ioex) {
		ioex.printStackTrace();
		} finally {
		if (fout != null)
			try { fout.close(); fout = null; }
			catch (Exception ex) {}
		}
	}

	public boolean compile() throws java.io.IOException
	{
		// write the source code if necesssary
		if (!isEncoded) 
		encode();

		if (!isCompiled) {
		VirtualCompiler compiler = vm.getCompiler();
		if (compiler == null) {
			String home = System.getProperty("java.home");
			//System.out.println("java.home: "+home);
			if (home == null) {
			compiler = new Javac();
			} else {
			// adjust for home
			if (home.endsWith("/jre")) {
				int len = home.length() - 4;
				home = home.substring(0, len);
			}			
			String toolsjar = home + "/lib/tools.jar";
			String rtjar = home + "/jre/lib/rt.jar";

			Jikes jikes = new Jikes();
			jikes.addClasspath(home + "/jre/lib/rt.jar");
			jikes.addClasspath(home + "/lib/tools.jar");
			compiler = jikes;
			}
		}
		isCompiled = compiler.compile(this);
		}

		return isCompiled;
	}

	public String getCodebase()
	{
		return codebase;
	}

	public Namespace getNamespace()
	{
		if (namespace == null)
		namespace = new MNamespace(vm, null);
		return namespace;
	}

	public TypeDeclaration getTopLevelType()
	{
		return types.isEmpty() ? null : (TypeDeclaration)types.elementAt(0);
	}

	public Namespace setNamespace(String name)
	{
		namespace = new MNamespace(vm, name);
		return namespace;
	}

	public Iterator getImports()
	{
		return new EnumerationIterator(imports.elements(), imports.size());
	}

	public Import addImport(String name)
	{
		int size = imports.size();
		
		for (int impLoop =0; impLoop < size; impLoop++) {
			MImport im = (MImport)imports.elementAt(impLoop);
			if (im.isClassnamePartOfImport(name))
				return im;
		}
		
		Import x = new MImport(vm, name);
		imports.addElement(x);
		return x;
	}

	public PackageClass getPackageClass(String name) {
		if (name == null)
			return null;
		Iterator it = getTypes();
		
		while(it.hasNext()) {
			Object obj = it.next();
			
			if (obj instanceof MPackageClass) {
				PackageClass pc = (PackageClass)obj;
				
				if (pc.getName().equals(name))
					return(pc);
			}
		}
		
		return null;
	}
	
	public PackageClass newClass(String name)
	{
		PackageClass x = new MPackageClass(vm, this);
		x.setName(name);
		types.addElement(x);
		return x;
	}

	public Interface newInterface(String name)
	{
		Interface x = new MInterface(vm, this);
		x.setName(name);
		types.addElement(x);
		return x;
	}

	public Iterator getTypes()
	{
		return new EnumerationIterator(types.elements(), types.size());
	}

	public CodeWriter toCode(CodeWriter out)
	{
		if (comment != null)
		out.write(comment).newLine();
	    
		if (namespace != null)
		out.write(namespace).newLine();

		Iterator iterator = getImports();
		if (iterator.size() > 0)
		out.write( getImports() );

		out.write( getTypes() );

		return out;
	}

	boolean isEncoded; // flag to track if we have encoded yet.
	boolean isCompiled; // flag to track if we have compiled yet.
	Vector types;
	Vector imports;
	String codebase;
	Namespace namespace;
	}

	// ===============================================================
	// PACKAGE DECLARATION
	// ===============================================================
	static class MNamespace
	extends MDeclaration
	implements Namespace
	{
	MNamespace(MVM vm, String name)
	{
		super(vm);
		this.name = name;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public String getName()
	{
		return name;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		if (!out.isLineNew()) out.newLine();
		super.toCode(out);
		out.write("package ").write(name).write(';').newLine();
		return out;
	}

	String name;
	}

	// ===============================================================
	// IMPORT DECLARATION
	// ===============================================================
	static class MImport
	extends MDeclaration
	implements Import
	{
	MImport(MVM vm, String name)
	{
		super(vm);
		this.name = name;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public String getName()
	{
		return name;
	}

	public boolean isSingle()
	{
		return !name.endsWith("*");
	}
	
	public boolean isClassnamePartOfImport(String fullclassname) {
		if (isSingle())
			return getName().equals(fullclassname);
		else {
			String packagename = getName().substring(0, getName().lastIndexOf("."));
			return packagename.equals(fullclassname.substring(0, fullclassname.lastIndexOf(".")));
		}
			
	}

	public CodeWriter toCode(CodeWriter out)
	{
		if (!out.isLineNew()) out.newLine();
		super.toCode(out);
		out.write("import ").write(name).write(';').newLine();
		return out;
	}

	String name;
	}

	// ===============================================================
	// MEMBER DECLARATION
	// ===============================================================
	abstract static class MMember
	extends MDeclaration
	implements Member
	{
	MMember(MVM vm)
	{
		super(vm);
		access = Access.PACKAGE;
		annotations = new Vector();
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public String getName()
	{
		return name;
	}

	public void isFinal(boolean value)
	{
		this.isFinal = value;
	}

	public boolean isFinal()
	{
		return isFinal;
	}

	public void isStatic(boolean value)
	{
		this.isStatic = value;
	}

	public boolean isStatic()
	{
		return isStatic;
	}

	public void setAccess(int access)
	{
		this.access = access;
	}

	public int getAccess()
	{
		return access;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		if (!out.isLineNew())
		out.newLine();

		out.newLine();

		if (comment != null)
		out.write(comment);
		if(annotations.size()>0){
			out.newLine();
			Iterator iterator = getAnnotations();
			while (iterator.hasNext()) {
				out.write((Annotation) iterator.next());
				out.newLine();
			}
		}
		if (!out.isLineNew()) 
		out.newLine();

		return out;
	}

	void writeBlock(CodeWriter out, BlockStyle style)
	{
		style.toCode(out, iterator());
	}



	public Iterator getAnnotations()
	{
		return new EnumerationIterator(annotations.elements(), annotations.size());
	}

	public Annotation addAnnotation(String text)
	{
		Annotation x = new MAnnotation(vm, text);
		annotations.addElement(x);
		return x;
	}
	
	public Annotation addAnnotation(String name, String text)
	{
		Annotation x = new MAnnotation(vm, name, text);
		annotations.addElement(x);
		return x;
	}

	public void addAnnotation(Annotation annotation)
	{
		annotations.addElement(annotation);
	}
	
	
	abstract Iterator iterator();

	String name;
	int access;
	boolean isFinal;
	boolean isStatic;
	Vector annotations;
	}

	// ===============================================================
	// MEMBER DECLARATION
	// ===============================================================
	abstract static class MTypeDeclaration
	extends MMember
	implements TypeDeclaration
	{
	MTypeDeclaration(MVM vm, MCompilationUnit unit)
	{
		super(vm);
		this.unit = unit;
		members = new Vector();
	}

	public Class load() throws ClassNotFoundException
	{
		synchronized (vm) {
		try {
			// set ourself as the loading class
			vm.loadingType = this;
			// make the class
			Class cls = vm.load(this);
			// clear the loading space
			vm.loadingType = null;
			// return the class
			return cls;
		} catch (ClassNotFoundException cnfex) {
			// clear the loading space
			vm.loadingType = null;
			// rethrow
			throw cnfex;
		}
		}
	}

	Codeable[] sort(Iterator iterator)
	{
		int len = iterator.size();
		Codeable[] a = new Codeable[len];
	    
		int i = 0;
		while (iterator.hasNext())
		a[i++] = (Codeable)iterator.next();
	    
		java.util.Arrays.sort(a, getComparator());
		return a;
	}

	protected Comparator getComparator()
	{
		if (comparator == null) {
		String className = System.getProperty("jenesis.member-comparator-classname");
		if (className != null) {
			try {
			comparator = (Comparator)Class.forName(className).newInstance();
			} catch (Exception ex) {
			System.out.println("Unable to load comparator: "+className);
			ex.printStackTrace();
			} 
		}
		if (comparator == null)
			comparator = new MemberComparator();
		}
		return comparator;
	}

	Iterator iterator()
	{
		return new ArrayIterator( sort( getMembers() ) );
	}

	public Iterator getMembers()
	{
		return new EnumerationIterator(members.elements(), members.size());
	}

	public CompilationUnit getUnit()
	{
		return unit;
	}

	/**
	 * Note:  this method does not take inner classes or fancy class names
	 * as of yet.
	 */
	String getClassfile()
	{
		// make the fully qualified class name
		StringBuffer className = new StringBuffer();

		// append to the filepath if exists (should, but
		// not required).
		if (unit.namespace != null) {
				// fetch the package name
		String pkg = unit.namespace.getName();
				// make sure the package name exists
		if (pkg != null) {
			// append the package name
			className.append(pkg).append('.');
		}
		}

		// add the name
		className.append(this.getName());

		// done
		return className.toString();
	}

	String getClassfileName()
	{
		// make the fileName of the target file as the concatenation
		// of the sourcepath, the package name, and the className,
		// and the ending dot class
		StringBuffer file = null;

		// was a path specified?
		if (unit.codebase != null && unit.codebase.length() > 0) {
				// make the buffer
		file = new StringBuffer(unit.codebase);
				// does the codebase end with a slash?
		if (!unit.codebase.endsWith(new Character(File.separatorChar).toString()))
			// make it so...
			file.append(File.separatorChar);
		} else {
		file = new StringBuffer();
		}

		// append to the filepath if exists
		if (unit.namespace != null) {
				// fetch the package name
		String pkg = unit.namespace.getName();
				// make sure the package name exists
		if (pkg != null) {
			// append the package name
			file.append(pkg.replace('.', java.io.File.separatorChar));
		}
		}

		// add a separator, the name, and the extention
		file.append(java.io.File.separatorChar)
		.append(this.getName())
		.append(".class");

		// done
		return file.toString();
	}

	MCompilationUnit unit;
	Vector members;
	static Comparator comparator;
	}

	// ===============================================================
	// INTERFACE DECLARATION
	// ===============================================================
	static class MInterface
	extends MTypeDeclaration
	implements Interface
	{
	MInterface(MVM vm, MCompilationUnit unit)
	{
		super(vm, unit);
		extendz = new Vector();
	}

	public Iterator getExtends()
	{
		return new EnumerationIterator(extendz.elements(), extendz.size());
	}

	public Interface addExtends(String ex)
	{
		extendz.addElement(ex);
		return this;
	}

	public Constant newConstant(Type type, String name)
	{
		Constant x = new MConstant(vm, type, name);
		members.addElement(x);
		return x;
	}

	public Constant newConstant(String name, int val)
	{
		Constant x = new MConstant(vm, vm.INT, name);
		x.setExpression(new MLiteral.MIntLiteral(vm, val));
		members.addElement(x);
		return x;
	}

	public AbstractMethod newMethod(Type type, String name)
	{
		AbstractMethod x = new MAbstractMethod(vm, type, name);
		members.addElement(x);
		return x;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		super.toCode(out);

		switch (access) {
		case Access.PUBLIC: out.write("public "); break;
		case Access.PRIVATE: out.write("private "); break;
		case Access.PROTECTED: out.write("protected "); break;
		}

		if (isFinal) out.write("final ");
		out.write("interface ").write(name);

		if (extendz.size() > 0) {
		out.newLine().write("extends ");

		for (int i=0; i < extendz.size(); i++) {
			if (i>0) out.write(", ");
			out.write(extendz.elementAt(i));
		}
		}

		writeBlock(out, vm.getStyle("interface"));
		return out;
	}

	// the vector of extends clauses
	Vector extendz;
	}

	// ===============================================================
	// CLASS DECLARATION
	// ===============================================================
	static class MClassDeclaration
	extends MTypeDeclaration
	implements ClassDeclaration
	{
	MClassDeclaration(MVM vm, MCompilationUnit unit)
	{
		super(vm, unit);
		implementations = new Vector();
	}

	public void isAbstract(boolean value)
	{
		this.isAbstract = value;
	}

	public boolean isAbstract()
	{
		return isAbstract;
	}

	public void setExtends(String ex)
	{
		this.ex = ex;
	}

	public String getExtends()
	{
		return ex;
	}

	public void addImplements(String type)
	{
		implementations.addElement(type);
	}

	public Iterator getImplements()
	{
		return new EnumerationIterator(implementations.elements(), implementations.size());
	}

	public Iterator getInitializations()
	{
		return new TypeIterator(InitializationDeclaration.class, members.elements(), members.size());
	}

	public Iterator getConstructors()
	{
		return new TypeIterator(Constructor.class, members.elements(), members.size());
	}

	public Iterator getInnerClasses()
	{
		return new TypeIterator(InnerClass.class, members.elements(), members.size());
	}

	public Iterator getMembers()
	{
		return new TypeIterator(Member.class, members.elements(), members.size());
	}
	
	public boolean isMethodAlreadyDefined(Type type, String name) {
		Iterator it = getMethods();

		while(it.hasNext()) {
			ClassMethod cm = (ClassMethod)it.next();
			if (cm.getName().equals(name) && cm.getType().equals(type))
				return true;
		}
		return false;
	}
	

	public Iterator getStaticInitializers()
	{
		return new TypeIterator(StaticInitializer.class, members.elements(), members.size());
	}

	public Iterator getFields()
	{
		return new TypeIterator(ClassField.class, members.elements(), members.size());
	}

	public Iterator getMethods()
	{
		return new TypeIterator(ClassMethod.class, members.elements(), members.size());
	}

	public InnerClass newInnerClass(String name)
	{
		InnerClass x = new MInnerClass(vm, unit, this, name);
		members.addElement(x);
		return x;
	}

	public Constructor newConstructor()
	{
		Constructor x = new MConstructor(vm, name);
		members.addElement(x);
		return x;
	}

	public StaticInitializer newStaticInitializer()
	{
		StaticInitializer x = new MStaticInitializer(vm);
		members.addElement(x);
		return x;
	}

	public ClassField newField(Type type, String name)
	{
		ClassField x = new MClassField(vm, type, name);
		members.addElement(x);
		return x;
	}

	public ClassMethod newMethod(Type type, String name)
	{
		ClassMethod x = new MClassMethod(vm, type, name);
		members.addElement(x);
		return x;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		super.toCode(out);

		// write modifiers if needed
		switch (access) {
		case Access.PUBLIC: out.write("public "); break;
		case Access.PRIVATE: out.write("private "); break;
		case Access.PROTECTED: out.write("protected "); break;
		}

		if (isAbstract) out.write("abstract ");
		if (isStatic) out.write("static ");
		if (isFinal) out.write("final ");

		// write the class and name
		out.write("class ").write(name);

		// write the exnteds clause if ther is one
		if (ex != null) {
		out.newLine().write("extends ").write(ex);
		}

		// write the implements clause if there are any
		if (implementations.size() > 0) {
		out.newLine().write("implements ");
		for (int i=0; i<implementations.size(); i++) {
			if (i>0) out.write(", ");
			out.write(implementations.elementAt(i));
		}
		}
		writeBlock(out, vm.getStyle("class"));
		return out;
	}

	String ex;
	boolean isAbstract;
	Vector implementations;
	}

	// ===============================================================
	// PACKAGE CLASS DECLARATION
	// ===============================================================
	static class MPackageClass
	extends MClassDeclaration
	implements PackageClass
	{
	MPackageClass(MVM vm, MCompilationUnit unit)
	{
		super(vm, unit);
	}

	MCompilationUnit unit;
	}

	// ===============================================================
	// INNER CLASS DECLARATION
	// ===============================================================
	static class MInnerClass
	extends MClassDeclaration
	implements InnerClass
	{
	MInnerClass(MVM vm, MCompilationUnit unit, MClassDeclaration parent, String name)
	{
		super(vm, unit);
		this.parent = parent;
		this.name = name;
	}

	public ClassDeclaration getParentClass()
	{
			return parent;
	}

	MClassDeclaration parent;
	}

	// ===============================================================
	// LOCAL CLASS DECLARATION
	// ===============================================================
	static class MLocalClass
	extends MClassDeclaration
	implements LocalClass
	{
	MLocalClass(MVM vm, String name)
	{
		super(vm, null); /* * *| NOTE: we are passing null and thus cannot load() a local class! |* * */
		this.name = name;
	}

	public Comment comment(String text)
	{
		Comment sl = new MComment.MSingleLineComment(vm, text);
		this.comment = sl;
		return sl;
	}

	public void setLabel(String label)
	{
		this.label = label;
	}

	public String getLabel()
	{
		return label;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		super.toCode(out);

		out.write(";");

		return out;
	}

	String label;
	}

	// ===============================================================
	// FIELD DECLARATION
	// ===============================================================
	abstract static class MField
	extends MMember
	implements Field
	{
	public MField(MVM vm, Type type, String name)
	{
		super(vm);
		this.type = type;
		this.name = name;
	}

	public void setType(Type type)
	{
		this.type = type;
	}

	public Type getType()
	{
		return type;
	}

	public void setExpression(Expression expr)
	{
		this.expr = expr;
	}

	public Expression getExpression()
	{
		return expr;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		// We write fields in a special manner.  If a field has a
		// javadoc comment, separate it from the stuff above with
		// a newline.  If it does not have a comment or the
		// comment is not a javadoc, dont add another line.

		if (!out.isLineNew())
		out.newLine();

		if (comment != null && comment.type() == Comment.D)
		out.newLine().write(comment);
		
		if(annotations.size()>0){
			out.newLine();
			Iterator iterator = getAnnotations();
			while (iterator.hasNext()) {
				out.write((Annotation) iterator.next());
				out.newLine();
			}
		}
		
		if (!out.isLineNew()) 
		out.newLine();

		return out;
	}

	// never called
	Iterator iterator()
	{
		return null;
	}

	// the type
	Type type;
	// the init expr
	Expression expr;
	}

	// ===============================================================
	// CONSTANT DECLARATION
	// ===============================================================
	static class MConstant
	extends MField
	implements Constant
	{
	public MConstant(MVM vm, Type type, String name)
	{
		super(vm, type, name);
		this.isStatic = true;
		this.isFinal = true;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		super.toCode(out);

		// write the class and name
		out.write("static final ").write(type).space().write(name);

		if (expr != null)
		out.write(" = ").write(expr);

		out.write(';');
		return out;
	}
	}

	// ===============================================================
	// CLASS FIELD DECLARATION
	// ===============================================================
	static class MClassField
	extends MField
	implements ClassField
	{
	public MClassField(MVM vm, Type type, String name)
	{
		super(vm, type, name);
	}

	public void isTransient(boolean isTransient)
	{
		this.isTransient = isTransient;
	}

	public boolean isTransient()
	{
		return isTransient;
	}

	public void isVolatile(boolean isVolatile)
	{
		this.isVolatile = isVolatile;
	}

	public boolean isVolatile()
	{
		return isVolatile;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		super.toCode(out);

		// write modifiers if needed
		switch (access) {
		case Access.PUBLIC: out.write("public "); break;
		case Access.PRIVATE: out.write("private "); break;
		case Access.PROTECTED: out.write("protected "); break;
		}

		if (isTransient) out.write("transient ");
		if (isVolatile) out.write("volatile ");
		if (isStatic) out.write("static ");
		if (isFinal) out.write("final ");

		// write the class and name
		out.write(type).space().write(name);

		if (expr != null)
		out.write(" = ").write(expr);

		out.write(';');
		return out;
	}

	// the transient flag
	boolean isTransient;
	// the volatile flag
	boolean isVolatile;
	}

	// ===============================================================
	// BLOCK DECLARATION
	// ===============================================================
	static class BlockDeclaration
	extends MMember
	{
	BlockDeclaration(MVM vm)
	{
		super(vm);
		vs = new Vector();
	}

	public Iterator getStatements()
	{
		return new EnumerationIterator(vs.elements(), vs.size());
	}

	public Iterator iterator()
	{
		return getStatements();
	}

	public Empty newEmpty()
	{
		Empty x = new MStatement.MEmpty(vm);
		vs.addElement(x);
		return x;
	}

	public LocalBlock newLocalBlock()
	{
		LocalBlock x = new MStatement.MLocalBlock(vm);
		vs.addElement(x);
		return x;
	}

	public LocalClass newLocalClass(String name)
	{
		LocalClass x = new MLocalClass(vm, name);
		vs.addElement(x);
		return x;
	}

	public Break newBreak()
	{
		Break x = new MStatement.MBreak(vm);
		vs.addElement(x);
		return x;
	}

	public Continue newContinue()
	{
		Continue x = new MStatement.MContinue(vm);
		vs.addElement(x);
		return x;
	}

	public Return newReturn()
	{
		Return x = new MStatement.MReturn(vm);
		vs.addElement(x);
		return x;
	}

	public Let newLet(Type type)
	{
		Let x = new MStatement.MLet(vm, type);
		vs.addElement(x);
		return x;
	}

	public Throw newThrow(Expression e)
	{
		Throw x = new MStatement.MThrow(vm, e);
		vs.addElement(x);
		return x;
	}

	public While newWhile(Expression e)
	{
		While x = new MStatement.MWhile(vm, e);
		vs.addElement(x);
		return x;
	}

	public DoWhile newDoWhile(Expression e)
	{
		DoWhile x = new MStatement.MDoWhile(vm, e);
		vs.addElement(x);
		return x;
	}

	public For newFor()
	{
		For x = new MStatement.MFor(vm);
		vs.addElement(x);
		return x;
	}

	public Try newTry()
	{
		Try x = new MStatement.MTry(vm);
		vs.addElement(x);
		return x;
	}

	public Switch newSwitch(Expression e)
	{
		Switch x = new MStatement.MSwitch(vm, e);
		vs.addElement(x);
		return x;
	}

	public Synchronized newSynchronized(Expression e)
	{
		Synchronized x = new MStatement.MSynchronized(vm, e);
		vs.addElement(x);
		return x;
	}

	public If newIf(Expression e)
	{
		If x = new MStatement.MIf(vm, e);
		vs.addElement(x);
		return x;
	}

	public ExpressionStatement newStmt(Expression expr)
	{
		ExpressionStatement x = new MStatement.MExpressionStatement(vm, expr);
		vs.addElement(x);
		return x;
	}

	Vector vs;
	}

	// ===============================================================
	// METHOD DECLARATION
	// ===============================================================
	abstract static class MMethod
	extends BlockDeclaration
	implements Method
	{
	MMethod(MVM vm, Type type, String name)
	{
		super(vm);
		this.type = type;
		this.name = name;
		parameters = new Vector();
		throwz = new Vector();
	}

	public Iterator getParameters()
	{
		return new EnumerationIterator(parameters.elements(), parameters.size());
	}

	public FormalParameter addParameter(Type type, String name)
	{
		FormalParameter x = new MFormalParameter(vm, type, name);
		parameters.addElement(x);
		return x;
	}

	public Iterator getThrows()
	{
		// convert the vector contents to an array
		return new EnumerationIterator(throwz.elements(), throwz.size());
	}

	public void addThrows(String name)
	{
		throwz.addElement(name);
	}

	public void setType(Type type)
	{
		this.type = type;
	}

	public Type getType()
	{
		return type;
	}

	public boolean isAbstract()
	{
		return isAbstract;
	}

	public void isAbstract(boolean isAbstract)
	{
		this.isAbstract = isAbstract;
	}


	boolean isAbstract;
	Vector parameters;
	Vector throwz;
	Type type;
	}

	// ===============================================================
	// ABSTRACT METHOD DECLARATION
	// ===============================================================
	static class MAbstractMethod
	extends MMethod
	implements AbstractMethod
	{
	MAbstractMethod(MVM vm, Type type, String name)
	{
		super(vm, type, name);
		this.isAbstract = true;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		super.toCode(out);

		// write modifiers if needed
		switch (access) {
		case Access.PUBLIC: out.write("public "); break;
		case Access.PRIVATE: out.write("private "); break;
		case Access.PROTECTED: out.write("protected "); break;
		}

		out.write(type).space().write(name).write('(');

		for (int i=0; i<parameters.size(); i++) {
		if (i>0) out.write(", ");
		out.write(parameters.elementAt(i));
		}

		out.write(')');

		if (throwz.size() > 0) {
		out.newLine().write("throws ");
		for (int i=0; i<throwz.size(); i++) {
			if (i>0) out.write(", ");
			out.write(throwz.elementAt(i));
		}
		}

		out.write(';');

		return out;
	}
	}

	// ===============================================================
	// CLASS METHOD DECLARATION
	// ===============================================================
	static class MClassMethod
	extends MMethod
	implements ClassMethod
	{
	MClassMethod(MVM vm, Type type, String name)
	{
		super(vm, type, name);
	}

	public void isNative(boolean isNative)
	{
		this.isNative = isNative;
	}

	public boolean isNative()
	{
		return isNative;
	}

	public void isSynchronized(boolean isSynchronized)
	{
		this.isSynchronized = isSynchronized;
	}

	public boolean isSynchronized()
	{
		return isSynchronized;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		super.toCode(out);

		// write modifiers if needed
		switch (access) {
		case Access.PUBLIC: out.write("public "); break;
		case Access.PRIVATE: out.write("private "); break;
		case Access.PROTECTED: out.write("protected "); break;
		}

		if (isAbstract) out.write("abstract ");
		if (isNative) out.write("native ");
		if (isSynchronized) out.write("synchronized ");
		if (isStatic) out.write("static ");
		if (isFinal) out.write("final ");

		// write the class and name
		out.write(type).space().write(name).write('(');

		for (int i=0; i<parameters.size(); i++) {
		if (i>0) out.write(", ");
		out.write(parameters.elementAt(i));
		}

		out.write(')');

		if (throwz.size() > 0) {
		out.newLine().write("throws ");
		for (int i=0; i<throwz.size(); i++) {
			if (i>0) out.write(", ");
			out.write(throwz.elementAt(i));
		}
		}
		 if (isAbstract) 
			out.write(";");
		else
			writeBlock(out, vm.getStyle("method"));
		    

		return out;
	}

	boolean isNative;
	boolean isSynchronized;
	}

	// ===============================================================
	// CONSTRUCTOR DECLARATION
	// ===============================================================
	static class MConstructor
	extends MMethod
	implements Constructor
	{
	MConstructor(MVM vm, String name)
	{
		super(vm, null, name);
	}

	public boolean isAbstract()
	{
		return false;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		super.toCode(out);

		// write modifiers if needed
		switch (access) {
		case Access.PUBLIC: out.write("public "); break;
		case Access.PRIVATE: out.write("private "); break;
		case Access.PROTECTED: out.write("protected "); break;
		}

		// write the class and name
		out.write(name).write('(');

		for (int i=0; i<parameters.size(); i++) {
		if (i>0) out.write(", ");
		out.write(parameters.elementAt(i));
		}

		out.write(')');

		if (throwz.size() > 0) {
		out.newLine().write("throws ");
		for (int i=0; i<throwz.size(); i++) {
			if (i>0) out.write(", ");
			out.write(throwz.elementAt(i));
		}
		}

		writeBlock(out, vm.getStyle("constructor"));

		return out;
	}
	}

	// ===============================================================
	// STATIC INITIALIZATION DECLARATION
	// ===============================================================
	static class MStaticInitializer
	extends BlockDeclaration
	implements StaticInitializer
	{
	MStaticInitializer(MVM vm)
	{
		super(vm);
	}

	public CodeWriter toCode(CodeWriter out)
	{
		super.toCode(out);

		// write the class and name
		out.write("static");
		writeBlock(out, vm.getStyle("static-initializer"));
		return out;
	}
	}

	// ===============================================================
	// FORMAL PARAMETER DECLARATION
	// ===============================================================
	static class MFormalParameter
	extends MDeclaration
	implements FormalParameter
	{
	MFormalParameter(MVM vm, Type type, String name)
	{
		super(vm);
		this.type = type;
		this.name = name;
	}

	public void setType(Type type)
	{
		this.type = type;
	}

	public Type getType()
	{
		return type;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public String getName()
	{
		return name;
	}

	public void isFinal(boolean value)
	{
		this.isFinal = value;
	}

	public boolean isFinal()
	{
		return isFinal;
	}

	public CodeWriter toCode(CodeWriter out)
	{
		out.queue(comment);
		if (isFinal) out.write("final ");
		out.write(type).space().write(name);
		return out;
	}

	Type type;
	String name;
	boolean isFinal;
	}
	MDeclaration(MVM vm)
	{
	super(vm);
	}
	public DocumentationComment javadoc(String text)
	{
	this.comment = new MComment.MDocumentationComment(vm, text);
	return (DocumentationComment)comment;
	}
}
