package com.inxar.jenesis;

/**
 * $Id: MStatement.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
import java.util.Vector;
import org.inxar.jenesis.*;
import com.inxar.jenesis.util.*;

/**
 * Standard <code>Statement</code> implementations.
 */
public abstract class MStatement
	extends MVM.MCodeable
	implements Statement
{
	String label;

	// ===============================================================
	// EMPTY STATEMENT
	// ===============================================================
	static class MEmpty
	extends MStatement
	implements Empty
	{
	MEmpty(MVM vm)
	{
	    super(vm);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write(';');
	    return out;
	}
	}

	// ===============================================================
	// GOTO STATMENTS
	// ===============================================================
	abstract static class GotoStatement
	extends MStatement
	{
	GotoStatement(MVM vm, String kwd)
	{
	    super(vm);
	    this.kwd = kwd;
	}

	public void setGoto(String target)
	{
	    this.target = target;
	}

	public String getGoto()
	{
	    return target;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);

	    out.write(kwd);

	    if (target != null)
		out.space().write(target);

	    return out.write(';');
	}

	String kwd;
	String target;
	}

	// ===============================================================
	// BREAK STATEMENT
	// ===============================================================
	static class MBreak
	extends GotoStatement
	implements Break
	{
	MBreak(MVM vm)
	{
	    super(vm, "break");
	}
	}

	// ===============================================================
	// CONTINUE STATEMENT
	// ===============================================================
	static class MContinue
	extends GotoStatement
	implements Continue
	{
	MContinue(MVM vm)
	{
	    super(vm, "continue");
	}
	}

	// ===============================================================
	// EXPRESSIONABLE STATEMENT
	// ===============================================================
	abstract static class MExpressionableStatement
	extends MStatement
	{
	MExpressionableStatement(MVM vm)
	{
	    super(vm);
	}

	public void setExpression(Expression e)
	{
	    this.e = e;
	}

	public Expression getExpression()
	{
	    return e;
	}

	Expression e;
	}

	// ===============================================================
	// EXPRESSION STATEMENT
	// ===============================================================
	static class MExpressionStatement
	extends MExpressionableStatement
	implements ExpressionStatement
	{
	MExpressionStatement(MVM vm)
	{
	    super(vm);
	}

	MExpressionStatement(MVM vm, Expression e)
	{
	    super(vm);
	    this.e = e;
	}

	public void setExpression(Expression e)
	{
	    this.e = e;
	}

	public Expression getExpression()
	{
	    return e;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);

	    out.write(e).write(';');

	    return out;
	}

	Expression e;
	}

	// ===============================================================
	// RETURN STATEMENT
	// ===============================================================
	static class MReturn
	extends MExpressionableStatement
	implements Return
	{
	MReturn(MVM vm)
	{
	    super(vm);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);

	    out.write("return");

	    if (e != null)
		out.space().write(e);

	    out.write(';');

	    return out;
	}
	}

	// ===============================================================
	// LET STATEMENT
	// ===============================================================
	static class MLet
	extends MExpressionableStatement
	implements Let
	{
	MLet(MVM vm, Type type)
	{
	    super(vm);
	    this.type = type;
	    this.v = new Vector();
	}

	public void setType(Type type)
	{
	    this.type = type;
	}

	public Type getType()
	{
	    return type;
	}

	public boolean isFinal()
	{
	    return isFinal;
	}

	public void isFinal(boolean value)
	{
	    this.isFinal = value;
	}

	public void addAssign(String name, Expression expr)
	{
	    v.addElement(
			 new MExpression.MAssign(vm, Assign.S, new MExpression.MVariable(vm, name), expr)
			     );
	}

	public Iterator getAssigns()
	{
	    return new EnumerationIterator(v.elements(), v.size());
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);

	    if (isFinal)
		out.write("final").space();

	    out.write(type).space();

	    for (int i=0; i<v.size(); i++) {
		if (i>0) out.write(',').space();
		out.write(v.elementAt(i));
	    }

	    out.write(';');

	    return out;
	}

	Type type;
	Variable var;
	boolean isFinal;
	Vector v;
	}

	// ===============================================================
	// THROW STATEMENT
	// ===============================================================
	static class MThrow
	extends MStatement
	implements Throw
	{
	MThrow(MVM vm, Expression e)
	{
	    super(vm);
	    this.e = e;
	}

	public void setThrowable(Expression e)
	{
	    this.e = e;
	}

	public Expression getThrowable()
	{
	    return e;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);

	    out.write("throw ").write(e).write(';');

	    return out;
	}

	Expression e;
	}

	// ===============================================================
	// BLOCK STATEMENT
	// ===============================================================
	static class BlockStatement
	extends MStatement
	implements Block
	{
	BlockStatement(MVM vm)
	{
	    super(vm);
	    // initialize the statments vector
	    vs = new Vector();
	}

	public Iterator getStatements()
	{
	    // convert the vector contents to an array
	    return new EnumerationIterator(vs.elements(), vs.size());
	}

	public Empty newEmpty()
	{
	    Empty x = new MEmpty(vm);
	    vs.addElement(x);
	    return x;
	}

	public LocalClass newLocalClass(String name)
	{
	    LocalClass x = new MDeclaration.MLocalClass(vm, name);
	    vs.addElement(x);
	    return x;
	}

	public LocalBlock newLocalBlock()
	{
	    LocalBlock x = new MLocalBlock(vm);
	    vs.addElement(x);
	    return x;
	}

	public Break newBreak()
	{
	    Break x = new MBreak(vm);
	    vs.addElement(x);
	    return x;
	}

	public Continue newContinue()
	{
	    Continue x = new MContinue(vm);
	    vs.addElement(x);
	    return x;
	}

	public Return newReturn()
	{
	    Return x = new MReturn(vm);
	    vs.addElement(x);
	    return x;
	}

	public Let newLet(Type type)
	{
	    Let x = new MLet(vm, type);
	    vs.addElement(x);
	    return x;
	}

	public Throw newThrow(Expression throwable)
	{
	    Throw x = new MThrow(vm, throwable);
	    vs.addElement(x);
	    return x;
	}

	public While newWhile(Expression predicate)
	{
	    While x = new MWhile(vm, predicate);
	    vs.addElement(x);
	    return x;
	}

	public DoWhile newDoWhile(Expression predicate)
	{
	    DoWhile x = new MDoWhile(vm, predicate);
	    vs.addElement(x);
	    return x;
	}

	public For newFor()
	{
	    For x = new MFor(vm);
	    vs.addElement(x);
	    return x;
	}

	public Try newTry()
	{
	    Try x = new MTry(vm);
	    vs.addElement(x);
	    return x;
	}

	public Switch newSwitch(Expression integer)
	{
	    Switch x = new MSwitch(vm, integer);
	    vs.addElement(x);
	    return x;
	}

	public Synchronized newSynchronized(Expression mutex)
	{
	    Synchronized x = new MSynchronized(vm, mutex);
	    vs.addElement(x);
	    return x;
	}

	public If newIf(Expression predicate)
	{
	    If x = new MIf(vm, predicate);
	    vs.addElement(x);
	    return x;
	}

	public ExpressionStatement newStmt(Expression expr)
	{
	    ExpressionStatement x = new MExpressionStatement(vm, expr);
	    vs.addElement(x);
	    return x;
	}

	protected void writeBlock(CodeWriter out, BlockStyle style)
	{
	    // write it
	    style.toCode(out, getStatements());
	}

	// the vector of statements
	Vector vs;
	}

	// ===============================================================
	// CONDITIONAL STATEMENT
	// ===============================================================
	abstract static class MConditionalStatement
	extends BlockStatement
	implements ConditionalStatement
	{
	public MConditionalStatement(MVM vm, Expression e)
	{
	    super(vm);
	    this.e = e;
	}

	public MConditionalStatement(MVM vm)
	{
	    super(vm);
	}

	public void setPredicate(Expression e)
	{
	    this.e = e;
	}

	public Expression getPredicate()
	{
	    return e;
	}

	Expression e;
	}

	// ===============================================================
	// LOCAL BLOCK STATEMENT
	// ===============================================================
	static class MLocalBlock
	extends BlockStatement
	implements LocalBlock
	{
	public MLocalBlock(MVM vm)
	{
	    super(vm);
	    this.blockStyle = blockStyle;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    this.writeBlock(out, vm.getStyle("local-class"));
	    return out;
	}

	String blockStyle;
	}

	// ===============================================================
	// SYNCHRONIZED STATEMENT
	// ===============================================================
	static class MSynchronized
	extends BlockStatement
	implements Synchronized
	{
	MSynchronized(MVM vm, Expression mutex)
	{
	    super(vm);
	    this.mutex = mutex;
	}

	public void setMutex(Expression mutex)
	{
	    this.mutex = mutex;
	}

	public Expression getMutex()
	{
	    return mutex;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("synchronized ").write('(').write(mutex).write(')');
	    this.writeBlock(out, vm.getStyle("synchronized"));
	    return out;
	}

	Expression mutex;
	}

	// ===============================================================
	// WHILE STATEMENT
	// ===============================================================
	static class MWhile
	extends MConditionalStatement
	implements While
	{
	public MWhile(MVM vm, Expression e)
	{
	    super(vm, e);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("while (").write(e).write(')');
	    this.writeBlock(out, vm.getStyle("while"));
	    return out;
	}
	}

	// ===============================================================
	// DO WHILE STATEMENT
	// ===============================================================
	static class MDoWhile
	extends MConditionalStatement
	implements DoWhile
	{
	public MDoWhile(MVM vm, Expression e)
	{
	    super(vm, e);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("do");
	    this.writeBlock(out, vm.getStyle("do"));
	    out.write("while (").write(e).write(");");
	    return out;
	}
	}

	// ===============================================================
	// FOR STATEMENT
	// ===============================================================
	static class MFor
	extends MConditionalStatement
	implements For
	{
	public MFor(MVM vm)
	{
	    super(vm);
	    // the vector of expression statements
	    vi = new Vector();
	    vu = new Vector();
	}

	public Let setInit(Type type)
	{
	    Let let = new MForInitLet(vm, type);
	    vi.addElement(let);
	    return let;
	}

	public Iterator getInits()
	{
	    return new EnumerationIterator(vi.elements(), vi.size());
	}

	public Iterator getUpdates()
	{
	    // convert the vector contents to an array
	    return new EnumerationIterator(vu.elements(), vu.size());
	}

	public void addInit(Expression e)
	{
	    vi.addElement(e);
	}

	public void addUpdate(Expression e)
	{
	    vu.addElement(e);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("for (");
	    write(vi, out);
	    out.write(';');
	    if (e != null) 
		out.space().write(e);
	    out.write(';');
	    if (vu.size() > 0) {
		out.space();
		write(vu, out);
	    }
	    out.write(')');
	    this.writeBlock(out, vm.getStyle("for"));
	    return out;
	}

	private void write(Vector v, CodeWriter out)
	{
	    for (int i=0; i<v.size(); i++) {
		if (i>0) out.write(", ");
		out.write(v.elementAt(i));
	    }
	}

	Vector vi;
	Vector vu;
	Let let;
	}

	// ===============================================================
	// LET STATEMENT
	// ===============================================================
	static class MForInitLet
	extends MLet
	{
	MForInitLet(MVM vm, Type type)
	{
	    super(vm, type);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    // queue up the comment
	    out.queue(comment);

	    if (isFinal) out.write("final").space();

	    out.write(type).space();

	    for (int i=0; i<v.size(); i++) {
		if (i>0) out.write(',').space();
		out.write(v.elementAt(i));
	    }

	    return out;
	}
	}


	// ===============================================================
	// SWITCH STATEMENT
	// ===============================================================
	static class MSwitch
	extends MConditionalStatement
	implements Switch
	{
	MSwitch(MVM vm, Expression e)
	{
	    super(vm, e);
	}

	public Iterator getCases()
	{
	    // convert the vector contents to an array
	    return new EnumerationIterator(vs.elements(), vs.size());
	}

	public Case newCase(Expression constant)
	{
	    Case c = new MCase(vm, constant);
	    vs.addElement(c);
	    return c;
	}

	public Default getDefault()
	{
	    if (d == null)
		d = new MDefault(vm);
	    return d;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    if (!triggered) {
				// add in the default if its not null
		if (d != null)
		    vs.addElement(d);
		triggered = true;
	    }

	    super.toCode(out);
	    out.write("switch (").write(e).write(')');
	    this.writeBlock(out, vm.getStyle("switch"));
	    return out;
	}

	boolean triggered;
	Default d;
	}

	// ===============================================================
	// CASE STATEMENT
	// ===============================================================
	static class MCase
	extends MConditionalStatement
	implements Case
	{
	MCase(MVM vm, Expression expr)
	{
	    super(vm);
	    setPredicate(expr);
	}

	public void setConstant(Expression expr)
	{
	    setPredicate(expr);
	}

	public Expression getConstant()
	{
	    return getPredicate();
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("case ").write(e).write(':');
	    this.writeBlock(out, vm.getStyle("case"));
	    return out;
	}
	}

	// ===============================================================
	// DEFAULT STATEMENT
	// ===============================================================
	static class MDefault
	extends BlockStatement
	implements Default
	{
	MDefault(MVM vm)
	{
	    super(vm);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("default:");
	    this.writeBlock(out, vm.getStyle("default"));
	    return out;
	}
	}

	// ===============================================================
	// IF STATEMENT
	// ===============================================================
	static class MIf
	extends MConditionalStatement
	implements If
	{
	MIf(MVM vm, Expression e)
	{
	    super(vm, e);
	    // the vector of clause else ifs
	    vcei = new Vector();
	}

	public Iterator getElseIfs()
	{
	    // convert the vector contents to an array
	    return new EnumerationIterator(vcei.elements(), vcei.size());
	}

	public ElseIf newElseIf(Expression e)
	{
	    ElseIf ei = new MElseIf(vm, e);
	    vcei.addElement(ei);
	    return ei;
	}

	public Else getElse()
	{
	    if (_else == null)
		_else = new MElse(vm);
	    return _else;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("if (").write(e).write(')');
	    this.writeBlock(out, vm.getStyle("if"));
	    codeElseIfs(vcei, out);
        if (_else != null && _else.getStatements().hasNext()) {
            out.write(_else);
        }
	    return out;
	}

	private CodeWriter codeElseIfs(Vector v, CodeWriter out)
	{
	    int len = v.size();

	    for (int i=0; i<len; i++)
		out.write((Codeable)v.elementAt(i));

	    return out;
	}

	Vector vcei;
	Else _else;
	}

	// ===============================================================
	// ELSE IF STATEMENT
	// ===============================================================
	static class MElseIf
	extends MConditionalStatement
	implements ElseIf
	{
	MElseIf(MVM vm, Expression e)
	{
	    super(vm, e);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("else if (").write(e).write(')');
	    this.writeBlock(out, vm.getStyle("else-if"));
	    return out;
	}
	}

	// ===============================================================
	// ELSE STATEMENT
	// ===============================================================
	static class MElse
	extends BlockStatement
	implements Else
	{
	MElse(MVM vm)
	{
	    super(vm);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("else");
	    this.writeBlock(out, vm.getStyle("else"));
	    return out;
	}
	}


	// ===============================================================
	// TRY STATEMENT
	// ===============================================================
	static class MTry
	extends BlockStatement
	implements Try
	{
	MTry(MVM vm)
	{
	    super(vm);
	    // the vector of clause else ifs
	    vcc = new Vector();
	}

	public Iterator getCatches()
	{
	    // convert the vector contents to an array
	    return new EnumerationIterator(vcc.elements(), vcc.size());
	}

	public Catch newCatch(Type type, String name)
	{
	    Catch cc = new MCatch(vm, type, name);
	    vcc.addElement(cc);
	    return cc;
	}

	public Finally getFinally()
	{
	    if (finallyClause == null)
		finallyClause = new MFinally(vm);
	    return finallyClause;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("try");
	    this.writeBlock(out, vm.getStyle("try"));
	    writeCatches(vcc, out);
	    out.write(finallyClause);
	    return out;
	}

	// utility method
	private CodeWriter writeCatches(Vector v, CodeWriter out)
	{
	    int len = v.size();

	    for (int i=0; i<len; i++)
		out.write((Codeable)v.elementAt(i));

	    return out;
	}

	Vector vcc;
	Finally finallyClause;
	}

	// ===============================================================
	// CATCH STATEMENT
	// ===============================================================
	static class MCatch
	extends BlockStatement
	implements Catch
	{
	MCatch(MVM vm, Type type, String name)
	{
	    super(vm);
	    this.fp = new MDeclaration.MFormalParameter(vm, type, name);
	}

	public void setThrowable(Type type, String name)
	{
	    this.fp = new MDeclaration.MFormalParameter(vm, type, name);
	}

	  	public FormalParameter getThrowable()
	{
	    return fp;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("catch (").write(fp).write(')');
	    this.writeBlock(out, vm.getStyle("catch"));
	    return out;
	}

	FormalParameter fp;
	}

	// ===============================================================
	// CATCH STATEMENT
	// ===============================================================
	static class MFinally
	extends BlockStatement
	implements Finally
	{
	MFinally(MVM vm)
	{
	    super(vm);
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    super.toCode(out);
	    out.write("finally");
	    this.writeBlock(out, vm.getStyle("finally"));
	    return out;
	}
	}
    public MStatement(MVM vm)
    {
	super(vm);
    }
    public Comment comment(String text)
    {
	Comment sl = new MComment.MSingleLineComment(vm, text);
	this.comment = sl;
	return sl;
    }
    public String getLabel()
    {
	return label;
    }
    public void setLabel(String label)
    {
	this.label = label;
    }
    public CodeWriter toCode(CodeWriter out)
    {
	// write the comment
	out.write(comment);

	// make a new line if this one is not new
	if (!out.isLineNew()) out.newLine();

	// and the label, if there is one.
	if (label != null) out.write(label).write(':').space();

	return out;
    }
}
