package com.inxar.jenesis;

/**
 * $Id: MStyle.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
import java.util.Enumeration;
import java.util.Properties;
import org.inxar.jenesis.*;
import com.inxar.jenesis.util.BlockStyle;
import com.inxar.jenesis.util.MemberComparator;

/**
 * Base class for the block style classes.
 */
abstract class MStyle
	implements BlockStyle
{
	static class Standard
	extends MStyle
	{
	public void toCode(CodeWriter out, Iterator iterator)
	{
	    if (!out.isLineNew()) 
		out.newLine();

	    if (iterator.size() > 0) {

		out.write('{')
		    .indentLine()
		    .write(iterator)
		    .dedentLine()
		    .write('}');

	    } else {

		out.write('{')
		    .newLine()
		    .write('}');

	    }
	}
	}

	static class SameLine
	extends MStyle
	{
	public void toCode(CodeWriter out, Iterator iterator)
	{
	    if (iterator.size() > 0) {

		out.space()
		    .write('{')
		    .indentLine()
		    .write(iterator)
		    .dedentLine()
		    .write('}').space();

	    } else {

		out.space()
		    .write('{')
		    .newLine()
		    .write('}');

	    }
	}
	}

	static class Optional
	extends MStyle
	{
	public void toCode(CodeWriter out, Iterator iterator)
	{
	    if (iterator.size() == 1) {

		out.indentLine()
		    .write(iterator)
		    .dedentLine();

	    } else {

		if (iterator.size() != 0) {
		    out.space()
			.write('{')
			.indentLine()
			.write(iterator)
			.dedentLine()
			.write('}');
		} else {
		    out.space()
			.write('{')
			.newLine()
			.write('}');
		}
	    }
	}
	}

	static class MStyleMap
	{
	MStyleMap(Properties p)
	{
	    this.p = p;
	    initStyles();
	    initMap();
	}

	void initStyles()
	{
	    // get all the keys
	    Enumeration e = p.propertyNames();

	    String key = null;
	    while (e.hasMoreElements()) {
				key = (String)e.nextElement();
				if (key.startsWith("style.")) {
					addStyle(key, p.getProperty(key));
				}
	    }
	}

	void initMap()
	{
	    // get all the keys
	    Enumeration e = p.propertyNames();

	    String key = null;
	    while (e.hasMoreElements()) {
				key = (String)e.nextElement();
				if (key.startsWith("style.")) {
					continue;
				}
		// remap the object
		//System.out.println("getting key "+key+", property "+p.getProperty(key));
		p.put(key, p.get(p.getProperty(key)));
	    }
	}

	void addStyle(String key, String className)
	{
			try {
				//System.out.println("putting class for "+key);
				p.put(key, Class.forName(className).newInstance());
			} catch (Exception ex) {
				ex.printStackTrace();
			}
	}

	BlockStyle get(String key)
	{
	    return (BlockStyle)p.get(key);
	}

	Properties p;
	}
    MStyle()
    {
    }
}
