package com.inxar.jenesis;

/**
 * $Id: MType.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
import org.inxar.jenesis.*;
import com.inxar.jenesis.util.*;

/**
 * Standard <code>Type</code> implementations.
 */
public abstract class MType
	extends MVM.MCodeable
{
	int type;
	String name;

	/* =============================================================== */
	/* PRIMITIVE TYPE */
	/* =============================================================== */
	static class MPrimitiveType
	extends MType
	implements PrimitiveType
	{
	MPrimitiveType(MVM vm, int type, String name)
	{
	    super(vm, type, name);
	}
	}

	/* =============================================================== */
	/* CLASS TYPE */
	/* =============================================================== */
	static class MClassType
	extends MType
	implements ClassType
	{
	MClassType(MVM vm, String name)
	{
	    super(vm, Type.CLASS, name);
	}

	public String getName()
	{
	    return name;
	}
	}

	/* =============================================================== */
	/* ARRAY TYPE */
	/* =============================================================== */
	static class MArrayType
	extends MType
	implements ArrayType
	{
	public MArrayType(MVM vm, Type type, int dims)
	{
	    super(vm, Type.ARRAY, type.toString());
	    this.type = type;
	    this.dims = dims;
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    out.queue(comment);
	    type.toCode(out);
	    for (int i=0; i<dims; i++)
		out.write("[]");
	    return out;
	}

	public Type getComponentType()
	{
	    return type;
	}

	public int getDims()
	{
	    return dims;
	}

	Type type;
	int dims;
	}
    public MType(MVM vm, int type, String name)
    {
	super(vm);
	this.type = type;
	this.name = name;
    }
    public boolean isArray()
    {
	return type == Type.ARRAY;
    }
    public boolean isPrimitive()
    {
	return type != Type.CLASS && type != Type.ARRAY;
    }
    public CodeWriter toCode(CodeWriter out)
    {
	out.queue(comment);
	out.write(name);
	return out;
    }
    public String toString()
    {
	return name;
    }
    public int type()
    {
	return type;
    }
}
