package com.inxar.jenesis;

/**
 * $Id: MVC.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
import java.io.*;
import java.util.Vector;
import java.util.StringTokenizer;
import sun.tools.javac.Main;
import org.inxar.jenesis.*;
import com.inxar.jenesis.*;
import com.inxar.jenesis.util.*;

/**
 * Base class for <code>VirtualCompiler</code> implementations.  
 */
public abstract class MVC
	implements VirtualCompiler
{
	protected String dest;
	protected Vector opts;
	protected Vector libs;
	protected Vector srcs;
    protected MVC()
    {
	this.libs = new Vector();
	this.srcs = new Vector();
	this.opts = new Vector();
    }
    public void addClasspath(String val)
    {
	libs.addElement(val);
    }
    public void addOption(String key)
    {
	opts.addElement(key);
    }
    public void addOption(String key, String val)
    {
	opts.addElement(key + " " + val);
    }
    public void addSourcepath(String val)
    {
	srcs.addElement(val);
    }
    public Iterator getClasspaths()
    {
	return new EnumerationIterator(libs.elements(), libs.size());
    }
    public String getDestination()
    {
	return dest;
    }
    public Iterator getOptions()
    {
	return new EnumerationIterator(opts.elements(), opts.size());
    }
    public Iterator getSourcepaths()
    {
	return new EnumerationIterator(srcs.elements(), opts.size());
    }
    protected void join(String[] array, String delim, StringBuffer buf)
    {
	// iterate simple loop
	for (int i=0; i<array.length; i++)
	    {
		if (i>0) buf.append(delim);
		buf.append(array[i]);
	    }
    }
    public void setDestination(String dest)
    {
	this.dest = dest;
    }
    protected String[] split(String s, String delim)
    {
	StringTokenizer st = new StringTokenizer(s, delim);
	String[] array = new String[st.countTokens()];
	int i=0;

	while (st.hasMoreTokens())
	    array[i++] = st.nextToken();

	return array;
    }
    protected String toFilename(CompilationUnit unit)
    {
	// fetch the codebase
	String codebase = unit.getCodebase();

	// make the fileName of the target file as the concatenation
	// of the sourcepath, the package name, and the className,
	// and the ending dot java
	StringBuffer file = null;

	// was a path specified?
	if (codebase != null && codebase.length() > 0) {
	    // make the buffer
	    file = new StringBuffer(codebase.replace('/', File.separatorChar));
	    // does the codebase end with a slash?
	    if (file.charAt(file.length()-1) != File.separatorChar) {
				// make it so...
		file.append(File.separatorChar);
	    }
	} else {
	    file = new StringBuffer();
	}

	// fetch the package name
	String pkg = unit.getNamespace().getName();

	// append to the filepath if exists
	if (pkg != null)
	    // append the package name
	    file.append(pkg.replace('.', File.separatorChar));

	// add a separator, the top level name, and the extention
	file.append(File.separatorChar)
	    .append(unit.getTopLevelType().getName())
	    .append(".java");

	return file.toString();
    }
}
