package com.inxar.jenesis;

/**
 * $Id: MVM.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
import java.io.*;
import java.util.Vector;
import java.util.Properties;
import org.inxar.jenesis.*;
import com.inxar.jenesis.util.BlockStyle;

/**
 * Standard <code>VirtualMachine</code> implementation.
 */
public class MVM
	extends org.inxar.jenesis.VirtualMachine
{
	PrimitiveType VOID;
	PrimitiveType NULL;
	PrimitiveType BOOLEAN;
	PrimitiveType BYTE;
	PrimitiveType SHORT;
	PrimitiveType INT;
	PrimitiveType CHAR;
	PrimitiveType LONG;
	PrimitiveType FLOAT;
	PrimitiveType DOUBLE;
	ClassType STRING;
	Null _NULL;
	True TRUE;
	False FALSE;

	MVMClassLoader loader;
	MStyle.MStyleMap styles;
	Vector units;
	MDeclaration.MTypeDeclaration loadingType;
	VirtualCompiler compiler;

	/* =============================================================== */
	/* CODEABLE SUPERPARENT */
	/* =============================================================== */
	abstract static class MCodeable
	implements Codeable
	{
	MCodeable(MVM vm)
	{
	    this.vm = vm;
	}

	public void setComment(int type, String text)
	{
	    switch (type) {
	    case Comment.S: comment = new MComment.MSingleLineComment(vm, text); break;
	    case Comment.M: comment = new MComment.MMultipleLineComment(vm, text); break;
		case Comment.Mbe: comment = new MComment.MMultipleLineComment(vm, text, true); break;
	    case Comment.D: comment = new MComment.MDocumentationComment(vm, text); break;
	    }
	}

	public CodeWriter toCode(CodeWriter out)
	{
	    if (comment != null)
		out.write(comment);
	    return out;
	}

	public Comment getComment()
	{
	    return comment;
	}

	public VirtualMachine vm()
	{
	    return vm;
	}

	public String toString()
	{
			java.io.StringWriter sout = new java.io.StringWriter();
			java.io.PrintWriter pout = new java.io.PrintWriter(sout);
			CodeWriter cout = new MCodeWriter(pout);
			this.toCode(cout);
			return sout.toString();
	}

	Comment comment;
	MVM vm;
	}

	/* =============================================================== */
	/* VM ClassLoader: JDK1.2 only right now!
	/* =============================================================== */
	class MVMClassLoader // not static.  needs parent references.
	extends ClassLoader
	{
	MVMClassLoader()
	{
	}

		/*
		 * Based on the class name alone we need to figure out which TypeDeclaration
		 * represents the class in question.  To assist in this process we use
		 * a field in the VM which anyone can write to which is a hint to the classloader
		 * that that type declaration shoud be used without having to search for it.
		 * This assumes correct implementation and management of this field such that
		 * the 'hint' will only be non-null when the TypeDeclaration iself was requested to
		 * be loaded.  In this way we should synchronize on the VM.
		 */
		protected Class findClass(String name) throws ClassNotFoundException
		{
	    //System.out.println(">> loading "+name);
	    // this method should only be called when a request on a TypeDeclaration occurred.
	    // Therefore, we should always be able to trust the the 'loadingType' field is valid.
	    File file = new File(loadingType.getClassfileName());
	    // get the length of the file
	    long len = file.length();
	    //System.out.println(">> loading file "+loadingType.getClassfileName());
	    // will it fit?  We can't be losing precision
	    if (len > Integer.MAX_VALUE)
		throw new ClassNotFoundException("Class file '"+loadingType.getClassfileName()+"' is too big! ["+len+" bytes]");
	    // make a new array of this length
	    byte[] bytes = new byte[(int)len];
	    // stream ref here in case we bail inside
	    BufferedInputStream in = null;

	    // now open a file and fetch get all the bytes into an array
	    try {
				// open a buffered fileinput stream such that
				// it will slurp it all at once...
		in = new BufferedInputStream( new FileInputStream(file), (int)len );
				// read in all the bytes
		in.read(bytes);
				// done
		in.close(); in = null;

	    } catch (IOException ioex) {
		// make sure stream is closed
		if (in != null)
		    try { in.close(); in = null; } catch (Exception ex) {}
		// rethrow new exception
		throw new ClassNotFoundException(ioex.toString());
	    }

	    // where we will create the class
	    Class cls = null;
	    // now we want to read int
	    cls = defineClass(name, bytes, 0, bytes.length);
	    // paranoid test?
	    if (cls == null) throw new ClassFormatError();
	    // resolve the class (what does this do anyway?)
	    resolveClass(cls);
	    // done
	    return cls;
		}
	}
    public MVM(String styleprops) throws IOException
    {
	BufferedInputStream in = null;
	Properties p = null;

	try {

	    // make the stream
	    in = new BufferedInputStream( new FileInputStream(styleprops) );
	    // fill the props
	    p = new Properties(); p.load(in);
	    // close stream
	    in.close(); in = null;

	} catch (IOException ioex) {
	    if (in != null)
		try { in.close(); } catch (Exception ex) {}
	    throw ioex;
	}

	init(p);
    }
    public MVM(java.util.Properties styleprops)
    {
	init(styleprops);
    }
    public void encode() throws java.io.IOException
    {
	for (int i=0; i<units.size(); i++) {
	    CompilationUnit c = (CompilationUnit)units.elementAt(i);
	    c.encode();
	}
    }
    public VirtualCompiler getCompiler()
    {
	return compiler;
    }
    BlockStyle getStyle(String name)
    {
	return styles.get(name);
    }
    void init(java.util.Properties styleprops)
    {
	initTypes();
	this.styles = new MStyle.MStyleMap(styleprops);
	this.units = new Vector();
    }
    void initTypes()
    {
	VOID = new MType.MPrimitiveType(this, Type.VOID, "void");
	NULL = new MType.MPrimitiveType(this, Type.NULL, "null");
	BOOLEAN = new MType.MPrimitiveType(this, Type.BOOLEAN, "boolean");
	BYTE = new MType.MPrimitiveType(this, Type.BYTE, "byte");
	SHORT = new MType.MPrimitiveType(this, Type.SHORT, "short");
	INT = new MType.MPrimitiveType(this, Type.INT, "int");
	CHAR = new MType.MPrimitiveType(this, Type.CHAR, "char");
	LONG = new MType.MPrimitiveType(this, Type.LONG, "long");
	FLOAT = new MType.MPrimitiveType(this, Type.FLOAT, "float");
	DOUBLE = new MType.MPrimitiveType(this, Type.DOUBLE, "double");
	STRING = new MType.MClassType(this, "String");
	_NULL = new MLiteral.MNull(this);
	TRUE = new MLiteral.MTrue(this);
	FALSE = new MLiteral.MFalse(this);
    }
    Class load(MDeclaration.MTypeDeclaration type)
	throws ClassNotFoundException
    {
        // make sure we have a classloader
        if (loader == null)
	    loader = new MVMClassLoader();

        // set a flag
        boolean ok = false;

        // make sure the type has been written to compiled
        if (!type.unit.isCompiled) {
	    try {
		ok = type.getUnit().compile();
	    } catch (Exception ioex) {
		throw new ClassNotFoundException("Cannot compile class:" + ioex.toString());
	    }
        } else {
	    ok = true;
        }

	// now try to load the class
	if (ok) {
	    // reconstruct the classname and delegate to classloader
            return loader.loadClass(type.getClassfile());
	} else {
	    throw new ClassNotFoundException("Cannot compile class.");
	}
    }
    public FieldAccess newAccess(String qual, String name)
    {
	return new MExpression.MFieldAccess(this, qual, name);
    }
    public NewAnonymousClass newAnon(Type type)
    {
	return new MExpression.MNewAnonymousClass(this, type);
    }
    public Annotation newAnnotation(String text)
    {
	return new MAnnotation(this, text);
    }
    public Annotation newAnnotation(String name, String text)
    {
	return new MAnnotation(this, name, text);
    }
    public ArrayType newArray(int type, int name)
    {
	return new MType.MArrayType(this, newType(type), name);
    }
    public ArrayType newArray(String type, int dims)
    {
	return new MType.MArrayType(this, newType(type), dims);
    }
    public NewArray newArray(Type type)
    {
	return new MExpression.MNewArray(this, type);
    }
    public ArrayType newArray(Type type, int dims)
    {
	return new MType.MArrayType(this, type, dims);
    }
    public ArrayAccess newArrayAccess(String qual, String name)
    {
	return new MExpression.MArrayAccess(this, qual, name);
    }
    public ArrayInitializer newArrayInit(Object o)
    {
	return new MExpression.MArrayInitializer(this, o);
    }
    public Assign newAssign(int type, Variable l, Expression r)
    {
	return new MExpression.MAssign(this, type, l, r);
    }
    public Assign newAssign(Variable l, Expression r)
    {
	return new MExpression.MAssign(this, Assign.S, l, r);
    }
    public Binary newBinary(int type, Expression l, Expression r)
    {
	return new MExpression.MBinary(this, type, l, r);
    }
    public Blank newBlank()
    {
	return new MExpression.MBlank(this);
    }
    public ByteLiteral newByte(byte val)
    {
	return new MLiteral.MByteLiteral(this, val);
    }
    public Cast newCast(Type type, Expression val)
    {
	return new MExpression.MCast(this, type, val);
    }
    public CharLiteral newChar(char val)
    {
	return new MLiteral.MCharLiteral(this, val);
    }
    public ClassLiteral newClass(String val)
    {
	return new MLiteral.MClassLiteral(this, newType(val));
    }
    public NewClass newClass(Type type)
    {
	return new MExpression.MNewClass(this, type);
    }
    public CompilationUnit newCompilationUnit(String path)
    {
	CompilationUnit x = new MDeclaration.MCompilationUnit(this, path);
	units.addElement(x);
	return x;
    }
    public DoubleLiteral newDouble(double val)
    {
	return new MLiteral.MDoubleLiteral(this, val);
    }
    public False newFalse()
    {
	return FALSE;
    }
    public FloatLiteral newFloat(float val)
    {
	return new MLiteral.MFloatLiteral(this, val);
    }
    public Freeform newFree(String code)
    {
	return new MExpression.MFreeform(this, code);
    }
    public IntLiteral newInt(int val)
    {
	return new MLiteral.MIntLiteral(this, val);
    }
    public Invoke newInvoke(String qual, String name)
    {
	return new MExpression.MInvoke(this, qual, name);
    }
    public LongLiteral newLong(long val)
    {
	return new MLiteral.MLongLiteral(this, val);
    }
    public Null newNull()
    {
	return _NULL;
    }
    public OctalLiteral newOctal(char val)
    {
	return new MLiteral.MOctalLiteral(this, val);
    }
    public ScientificLiteral newScientific(int precision, int scale, int exponent)
    {
	return new MLiteral.MScientificLiteral(this, precision, scale, exponent);
    }
    public ShortLiteral newShort(short val)
    {
	return new MLiteral.MShortLiteral(this, val);
    }
    public StringLiteral newString(String val)
    {
	return new MLiteral.MStringLiteral(this, val);
    }
    public Ternary newTernary(int type, Expression one, Expression two, Expression three)
    {
	return new MExpression.MTernary(this, type, one, two, three);
    }
    public True newTrue()
    {
	return TRUE;
    }
    public PrimitiveType newType(int type)
    {
	switch (type) {
	case Type.VOID: return VOID;
	case Type.NULL: return NULL;
	case Type.BOOLEAN: return BOOLEAN;
	case Type.BYTE: return BYTE;
	case Type.SHORT: return SHORT;
	case Type.INT: return INT;
	case Type.LONG: return LONG;
	case Type.FLOAT: return FLOAT;
	case Type.DOUBLE: return DOUBLE;
	case Type.CHAR: return CHAR;
	}
	return null;
    }
    public ClassType newType(String name)
    {
	return new MType.MClassType(this, name);
    }
    public Unary newUnary(int type, Expression val)
    {
	return new MExpression.MUnary(this, type, val);
    }
    public UnicodeLiteral newUnicode(char val)
    {
	return new MLiteral.MUnicodeLiteral(this, val);
    }
    public Variable newVar(String name)
    {
	return new MExpression.MVariable(this, name);
    }
    public void setCompiler(VirtualCompiler compiler)
    {
	this.compiler = compiler;
    }
}
