package org.inxar.jenesis;

/**
 * $Id: ArrayInitializer.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * <code>Expression</code> subinterface for array initializers.  The
 * array initialization expression has a few interesting features.
 * Notice, the argument is <code>Object</code>.  The contract is that
 * the <code>Object</code> must be of runtime-type
 * <code>Expression</code> or, more likely, <code>Object[]</code>.  If
 * the type is <code>Object[]</code>, then each element of that array
 * will recursively be introspected, checking again to see if the type
 * is <code>Expression</code> or <code>Object[]</code>.  In this
 * manner, one can arbitratily nest expressions.  This is the only way
 * I know how to accomplish this behavior (ie not knowing the
 * dimensionality of an argument at compile-time).
 * 
 * <P>
 *
 * For example, say we wanted to make the array:
 * 
 * <PRE>
 * int[][] aai = { {1, 2} , {3, 4} };
 * </PRE>
 * One could do this by:
 * <PRE>
 * Expression[] a1 = new Expression[2];
 * a1[0] = new IntLiteralImpl(1);
 * a1[1] = new IntLiteralImpl(2);
 * Expression[] a2 = new Expression[2];
 * a2[0] = new IntLiteralImpl(3);
 * a2[1] = new IntLiteralImpl(4);
 * Object[] a3 = new Object[2];
 * a3[0] = a1;
 * a3[1] = a2;
 * ArrayDeclarationStatement asd = new ArrayDeclarationStatementImpl(); // fictional implementation
 * asd.setInitialization(a3);
 * </PRE> 
 */
public interface ArrayInitializer
extends Expression
{
    /**
     * Gets the array initialization expressions.
     */
    Object getArgs();
    /**
     * Sets the array initialization expressions as an arbitrarily
     * nested array of <code>Expression[]</code> <i>or</i> as a single
     * <code>Expression</code>.  
     */
    void setArgs(Object o);
}
