package org.inxar.jenesis;

/**
 * $Id: Assign.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * <code>Expression</code> subinterface for assignments.
 */
public interface Assign
	extends Binary
{
	/**
	 * Assignment type for simple: <code>(a = b)</code>.
	 */
	static final int S = 1;
	
	/**
	 * Assignment type for bitwise and: <code>(a &= b)</code>.
	 */
	static final int BAND = 3;
	
	/**
	 * Assignment type for bitwise or: <code>(a |= b)</code>.
	 */
	static final int BOR = 4;
	
	/**
	 * Assignment type for bitwise xor: <code>(a ^= b)</code>.
	 */
	static final int XOR = 5;

	/**
	 * Assignment type for bitwise left shift: <code>(a <<= b)</code>.
	 */
	static final int LEFT = 6;

	/**
	 * Assignment type for bitwise right shift: <code>(a >>= b)</code>.
	 */
	static final int RIGHT = 7;

	/**
	 * Assignment type for bitwise unsigned right shift: <code>(a >>>= b)</code>.
	 */
	static final int UNSIGNED = 8;

	/**
	 * Assignment type for arithmetic addition: <code>(a += b)</code>.
	 */
	static final int ADD = 9;

	/**
	 * Assignment type for arithmetix subtraction: <code>(a -= b)</code>.
	 */
	static final int SUB = 10;

	/**
	 * Assignment type for arithmetic multiplication: <code>(a *= b)</code>.
	 */
	static final int MUL = 11;

	/**
	 * Assignment type for arithmetic division: <code>(a /= b)</code>.
	 */
	static final int DIV = 12;

	/**
	 * Assignment type for arithmetic modulus: <code>(a %= b)</code>.
	 */
	static final int MOD = 13;

	/**
	 * Assignment type for string concatenation: <code>(a += b)</code>.
	 */
	static final int CAT = 21;

    /**
     * Gets the lvalue.
     */
    Variable getVariable();
    /**
     * Sets the lvalue.
     */
    void setVariable(Variable variable);
    /**
     * Returns the type of this assignment as one of the constants in this interface.
     */
    int type();
}
