package org.inxar.jenesis;

/**
 * $Id: Block.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * The <code>Block</code> superinterface. A <code>Block</code> is a
 * structure that holds Statements and typically delimited by braces.
 * <code>Block</code> acts as a factory for all <code>Statement</code>
 * objects.  
 */
public interface Block
	extends Codeable
{
    /**
     * Gets the list of statements as an iterator of
     * <code>Statement</code>.  
     */
    Iterator getStatements();
    /**
     * Adds a new <code>Break</code> statement to this block and
     * returns it.  
     */
    Break newBreak();
    /**
     * Adds a new <code>Continue</code> statement to this block and
     * returns it.  
     */
    Continue newContinue();
    /**
     * Adds a new <code>DoWhile</code> statement to this block and
     * returns it.  
     */
    DoWhile newDoWhile(Expression predicate);
    /**
     * Adds a new <code>Empty</code> statement to this block and
     * returns it.  
     */
    Empty newEmpty();
    /**
     * Adds a new <code>For</code> statement to this block and returns
     * it.  
     */
    For newFor();
    /**
     * Adds a new <code>If</code> statement to this block and returns
     * it.  
     */
    If newIf(Expression predicate);
    /**
     * Adds a new <code>Let</code> statement to this block for the
     * given type and returns it.  
     */
    Let newLet(Type type);
    /**
     * Adds a new <code>LocalBlock</code> statement to this block and
     * returns it.  
     */
    LocalBlock newLocalBlock();
    /**
     * Adds a new <code>LocalClass</code> statement to this block and
     * returns it.  
     */
    LocalClass newLocalClass(String name);
    /**
     * Adds a new <code>Return</code> statement to this block and
     * returns it.  
     */
    Return newReturn();
    /**
     * Adds a new <code>ExpressionStatement</code> statement to this
     * block on the given statement <code>Expression</code> and
     * returns it.  
     */
    ExpressionStatement newStmt(Expression expr);
    /**
     * Adds a new <code>Switch</code> statement to this block and
     * returns it.  
     */
    Switch newSwitch(Expression integer);
    /**
     * Adds a new <code>Synchronized</code> statement to this block
     * and returns it.  
     */
    Synchronized newSynchronized(Expression mutex);
    /**
     * Adds a new <code>Throw</code> statement to this block and
     * returns it.  
     */
    Throw newThrow(Expression throwable);
    /**
     * Adds a new <code>Try</code> statement to this block and returns
     * it.  
     */
    Try newTry();
    /**
     * Adds a new <code>While</code> statement to this block and
     * returns it.  
     */
    While newWhile(Expression predicate);
}
