package org.inxar.jenesis;

import java.util.Vector;

/**
 * $Id: ClassDeclaration.java,v 1.3 2003-09-16 09:31:00 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * <code>Declaration</code> subinterface for the class declaration heirarchy.
 */
public interface ClassDeclaration
	extends TypeDeclaration
{
    /**
     * Adds the given string to the list of implements clauses.  
     */
    void addImplements(String type);
    /**
     * Gets the list of constructors as an iterator of
     * <code>Constructor</code>.  
     */
    Iterator getConstructors();
    /**
     * Gets the extends clause.
     */
    String getExtends();
    /**
     * Gets the list of fields as an iterator of
     * <code>ClassField</code>.  
     */
    Iterator getFields();
    /**
     * Gets the list of implementation clauses as an iterator of
     * <code>String</code>.  
     */
    Iterator getImplements();
    /**
     * Gets the list of inner classes as an iterator of
     * <code>InnerClass</code>.  
     */
    Iterator getInnerClasses();
    /**
     * Gets the list of method as an iterator of
     * <code>ClassMethod</code>.  
     */
    Iterator getMethods();
    /**
     * Gets the list of static initializers as an iterator of
     * <code>StaticInitializer</code>.  
     */
    Iterator getStaticInitializers();
    /**
     * Getter method for the isAbstract flag.
     */
    boolean isAbstract();
    /**
     * Setter method for the isAbstract flag.
     */
    void isAbstract(boolean value);
    /**
     * Creates a new constructor in this class 
     */
    Constructor newConstructor();
    /**
     * Creates a new field in this class with the given type and name.
     */
    ClassField newField(Type type, String name);
    /**
     * Creates a new inner class in this class.  
     */
    InnerClass newInnerClass(String name);
    /**
     * Creates a new method in this class with the given type and
     * name.  
     */
    ClassMethod newMethod(Type type, String name);
    /**
     * Creates a new static initialization block in this class 
     */
    
	boolean isMethodAlreadyDefined(Type type, String name);
	
    
    StaticInitializer newStaticInitializer();
    /**
     * Sets the extends clause to the given string. A class can extend
     * only one type.  
     */
    void setExtends(String type);
}
