package org.inxar.jenesis;

/**
 * $Id: Codeable.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * <code>Codeable</code> is a core interface which nearly all other interfaces extend.
 * Objects that implement this interface are expected to know how to render themselves
 * as source code to a <code>CodeWriter</code>.  Any <code>Codeable</code> is be commentable as
 * well.
 */
public interface Codeable
{
	/**
	 * Gets the comment for this <code>Codeable</code> object.
	 */
	Comment getComment();
	/**
	 * Sets the comment to the given type and text.
	 */
	void setComment(int type, String text);
	/**
	 * Renders the codeable object to the given writer and returns it.
	 */
	CodeWriter toCode(CodeWriter out);
	/**
	 * Returns the virtual machine to which this <code>Codeable</code> object belongs.
	 */
	VirtualMachine vm();
}
