package org.inxar.jenesis;

/**
 * $Id: CompilationUnit.java,v 1.2 2004-07-28 10:06:10 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * <code>Declaration</code> subinterface for compilation units.
 * A compilation unit is the source code atom in a typical filesystem.
 * It consists of three parts, each of which is optional:
 * <UL>
 * <LI>A package declaration (7.4), giving the fully qualified name (6.7) of the package to which the compilation unit belongs
 * <LI>Iimport declarations (7.5) that allow types from other packages to be referred to using their simple names
 * <LI>Type declarations (7.6) of class and interface types
 * </UL>
 */
public interface CompilationUnit
extends Codeable
{
	/**
	 * Adds the given import declaration to the compilation unit.
	 */
	Import addImport(String name);
	/**
	 * Generates the source code file and compiles it.
	 */
	boolean compile() throws java.io.IOException;
	/**
	 * Generates the source code file.
	 */
	void encode() throws java.io.IOException;
	/**
	 * Returns the filesystem location where the compilation unit should
	 * be written.
	 */
	String getCodebase();
	/**
	 * Gets the list of imports as an iterator of <code>ImportDeclaration</code>
	 */
	Iterator getImports();
	PackageClass getPackageClass(String name);
	
	/**
	 * Returns the package name as a <code>Package</code>.
	 */
	Namespace getNamespace();
	/**
	 * Gets the top level class or interface.
	 */
	TypeDeclaration getTopLevelType();
	/**
	 * Returns a list of types members as an iterator of <code>TypeDeclaration</code>.
	 */
	Iterator getTypes();
	/**
	 * Adds a new class to this compilation unit.
	 */
	PackageClass newClass(String name);
	/**
	 * Adds a new interface to this compilation unit.
	 */
	Interface newInterface(String name);
	/**
	 * Sets the package name and returns the corresponding package declaration.
	 */
	Namespace setNamespace(String name);
}
