package org.inxar.jenesis;

/**
 * $Id: For.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * <code>Statement</code> subinterface for the <code>for</code>
 * construct.  It has an declaration statement, a conditional
 * statement, and an update statement (all of which are optional).  
 */
public interface For
extends ConditionalStatement
{
    /**
     * Adds this statement expression to the comma separated list of
     * initialization expressions.  
     */
    void addInit(Expression expr);
    /**
     * Adds this declaration statement to the list of updates.
     */
    void addUpdate(Expression update);
    /**
     * Gets the comma separated list of initialzers from this for
     * statement as an iterator of <code>Expression</code> OR a single
     * <code>Let</code> object (depending on how it was defined).  One
     * need check the RTTI to be sure.  
     */
    Iterator getInits();
    /**
     * Gets the comma separated list of updates from this for
     * statement as an iterator of <code>Expression</code>.  
     */
    Iterator getUpdates();
    /**
     * Sets this declaration statement to a <code>Let</code> statement
     * with the given <code>Type</code>.  If any initialization
     * expressions have previously been added through the
     * <code>addInit(Expression)</code> method, they will all be
     * overwritten.  This is because the Java Language Specification
     * allows the initialization section of the for statement to be a
     * <code>Let</code> expression.  
     */
    Let setInit(Type type);
}
