package org.inxar.jenesis;

/**
 * $Id: Freeform.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * <code>Expression</code> subinterface for freeform expressions.  The
 * freeform expression is a very general container expression for
 * (hopefully) exceptional instances in which it the api is too
 * difficult or cumbersome and one needs to inject a source code
 * directly.  This structure simply copies the text given in setCode.
 * No semicolon is appended.  
 */
public interface Freeform
	extends Expression
{
    /**
     * Decrements the tab and calls newLine()
     */
    Freeform dedentLine();
    /**
     * Gets the code for this expression.
     */
    String getCode();
    /**
     * Returns the current number of indentation levels.
     */
    int getIndentNumber();
    /**
     * Increments the tab and calls newLine()
     */
    Freeform indentLine();
    /**
     * Returns true if no characters have been written since the last
     * call of newLine().
     */
    boolean isLineNew();
    /**
     * Adds a the newLine string according to
     * <CODE>System.getProperty("line.separator")</CODE> and the line
     * is padded with the n tab characters where n is the number
     * returned by <code>getIndentNumber()</code>.  
     */
    Freeform newLine();
    /**
     * Resets the tab counter to zero and calls the newLine() method.  
     */
    Freeform resetLine();
    /**
     * Sets the code for this expression.
     */
    void setCode(String code);
    /**
     * Writes a single space.
     */
    Freeform space();
    /**
     * Writes an array of characters.
     */
    Freeform write(char[] chars);
    /**
     * Writes each element of the given object array.
     */
    Freeform write(Object[] ao);
    /**
     * Writes a single character.
     */
    Freeform write(char c);
    /**
     * Writes a double.
     */
    Freeform write(double d);
    /**
     * Writes a float.
     */
    Freeform write(float f);
    /**
     * Writes an integer.
     */
    Freeform write(int i);
    /**
     * Writes an object.
     */
    Freeform write(Object o);
    /**
     * Writes a string.
     */
    Freeform write(String s);
    /**
     * Writes a boolean.
     */
    Freeform write(boolean b);
}
