package org.inxar.jenesis;

/**
 * $Id: Ternary.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * <code>Expression</code> subinterface for expressions which have a
 * three operands.  The only ternary function in Java is the ternary
 * if-else construct.  
 */
public interface Ternary
	extends Expression
{
	/**
	 * Ternary function type for if-else: <code>(a ? b : c)</code>.
	 */
	static final int IF = 1;
	
    /**
     * Accessor method for the first operand.
     */
    Expression getValue1();
    /**
     * Accessor method for the second operand.
     */
    Expression getValue2();
    /**
     * Accessor method for the third operand.
     */
    Expression getValue3();
    /**
     * Mutator method for the first operand.
     */
    void setValue1(Expression e);
    /**
     * Mutator method for the second operand.
     */
    void setValue2(Expression e);
    /**
     * Mutator method for the third operand.
     */
    void setValue3(Expression e);
    /**
     * Returns the type of this binary function as one of the
     * constants in this interface.  
     */
    int type();
}
