package org.inxar.jenesis;

/**
 * $Id: Unary.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * <code>Expression</code> subinterface for expressions which have a
 * single operand.  
 */
public interface Unary
	extends Expression
{
	/**
	 * Unary function type for a group of parentheses:
	 * <code>(expr)</code>.  
	 */
	static final int GROUP = 0;
	
	/**
	 * Unary function type for a logical complement:
	 * <code>(!true)</code>.  
	 */
	static final int NOT = 1;
	
	/**
	 * Unary function type for bitwise complement: <code>(~1)</code>.
	 */
	static final int BITWISE_NOT = 2;
	
	/**
	 * Unary function type for arithmetic sign complement:
	 * <code>(-1)</code>.  
	 */
	static final int NEG = 3;
	
	/**
	 * Unary function type for arithmetic sign identity:
	 * <code>(+1)</code>.  
	 */
	static final int POS = 4;
	
	/**
	 * Unary function type for arithmetic ante-increment:
	 * <code>(++i)</code>.  
	 */
	static final int AI = 5;
	
	/**
	 * Unary function type for arithmetic post-increment:
	 * <code>(i++)</code>.  
	 */
	static final int PI = 6;
	
	/**
	 * Unary function type for arithmetic ante-decrement:
	 * <code>(--i)</code>.  
	 */
	static final int AD = 7;
	
	/**
	 * Unary function type for arithmetic post-decrement:
	 * <code>(i--)</code>.  
	 */
	static final int PD = 8;
	
    /**
     * Getter method for the operand.
     */
    Expression getValue();
    /**
     * Setter method for the operand.
     */
    void setValue(Expression e);
    /**
     * Returns the type of this unary function as one of the constants
     * in this interface.  
     */
    int type();
}
