package org.inxar.jenesis;

/**
 * $Id: VirtualCompiler.java,v 1.1 2003-08-25 12:31:54 pra Exp $
 *
 * Copyright (C) 2000, 2001  Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
/**
 * The <code>VirtualCompiler</code> interface abstracts an object
 * which is capable of compiling <code>CompilationUnit</code> objects.  
 */
public interface VirtualCompiler
{
    /**
     * Adds the string to the list of classpaths.  
     */
    void addClasspath(String value);
    /**
     * Adds a boolean option with the given name.
     */
    void addOption(String name);
    /**
     * Adds an option with the given name and value.
     */
    void addOption(String name, String value);
    /**
     * Adds the string to the list of sourcepaths.
     */
    void addSourcepath(String val);
    /**
     * Generates the source code file and compiles the result.  
     */
    boolean compile(CompilationUnit unit) throws java.io.IOException;
    /**
     * Gets the classpath settings as an iterator of
     * <code>String</code>.  
     */
    Iterator getClasspaths();
    /**
     * Gets the base destination where new classes should be written.
     */
    String getDestination();
    /**
     * Gets the sourcepath settings as an iterator of
     * <code>String</code>.  
     */
    Iterator getOptions();
    /**
     * Gets the sourcepath settings as an iterator of
     * <code>String</code>.  
     */
    Iterator getSourcepaths();
    /**
     * Sets the base destination where new classes should be written.
     */
    void setDestination(String dst);
}
